#File: automeasure.tcl
#Syscomp USB Oscilloscope GUI
#Automeasurement Procedures

#JG
#Copyright 2006 Syscomp Electronic Design
#www.syscompdesign.com

#This program is free software; you can redistribute it and/or
#modify it under the terms of the GNU General Public License as
#published by the Free Software Foundation; either version 2 of
#the License, or (at your option) any later verison.
#
#This program is distributed in the hope that it will be useful, but
#WITHOUT ANY WARRANTY; without even the implied warranty of
#MECHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See
#the GNU General Public License for more details.
#
#You should have received a copy of the GNU General Public License
#along with this program; if not, write to the Free Software
#Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301
#USA

#Auto Measurements
#---------------------
#This procedure manages the global variables associated
#with the automeasurement labels.  It calls helper procedures
#which actually perform the calculations for the automatic
#measurements, and updates the global variables accoringly.
proc autoMeasurements {} {
	global	dataARaw	\
			dataBRaw	\
			autoFreqA	\
			autoFreqB	\
			autoPeriodA	\
			autoPeriodB	\
			autoAverageA	\
			autoAverageB	\
			autoMaxA		\
			autoMaxB		\
			autoMinA		\
			autoMinB		\
			autoPkPkA		\
			autoPkPkB		\
			enableA		\
			enableB		\
			stepA		\
			stepB			\
			mathEnabled	\
			mathData		\
			autoAverageMath	\
			autoFreqMath	\
			autoPeriodMath	\
			autoMaxMath	\
			autoMinMath	\
			autoPkPkMath	\
			autoRMSA		\
			autoRMSB		\
			autoRMSMath
	
	if {$enableA} {
		#Measure average value, frequency and period for Channel A
		set temp [autoFrequency $dataARaw $stepA]
		set autoAverageA [lindex $temp 0]
		set autoFreqA [lindex $temp 1]
		set autoPeriodA [lindex $temp 2]
		#Measure max, min, and peak-peak
		set temp [autoAmplitude $dataARaw $stepA]
		set autoMaxA [lindex $temp 0]
		set autoMinA [lindex $temp 1]
		set autoPkPkA [lindex $temp 2]
		set autoRMSA [autoRMSVoltage $dataARaw $stepA]
	} else {
		#Channel is disabled, blank off the labels
		set autoAverageA ""
		set autoFreqA ""
		set autoPeriodA ""
		set autoMaxA ""
		set autoMinA ""
		set autoPkPkA ""
	}
	
	if {$enableB} {
		#Measure average value, frequency and period for Channel B
		set temp [autoFrequency $dataBRaw $stepB]
		set autoAverageB [lindex $temp 0]
		set autoFreqB [lindex $temp 1]
		set autoPeriodB [lindex $temp 2]
		#Measure max, min, and peak-peak
		set temp [autoAmplitude $dataBRaw $stepB]
		set autoMaxB [lindex $temp 0]
		set autoMinB [lindex $temp 1]
		set autoPkPkB [lindex $temp 2]
		set autoRMSB [autoRMSVoltage $dataBRaw $stepA]
	} else {
		#Channel is disabled, blank off the labels
		set autoAverageB ""
		set autoFreqB ""
		set autoPeriodB ""
		set autoMaxB ""
		set autoMinB ""
		set autoPkPkB ""
	}
	
	if {$mathEnabled} {
		#Measure average value, frequency and period for math channel
		set temp [autoFrequency $mathData -1]
		set autoAverageMath [lindex $temp 0]
		set autoFreqMath [lindex $temp 1]
		set autoPeriodMath [lindex $temp 2]
		#Measure max, min, and peak-peak
		set temp [autoAmplitude $mathData -1]
		set autoMaxMath [lindex $temp 0]
		set autoMinMath [lindex $temp 1]
		set autoPkPkMath [lindex $temp 2]
		set autoRMSMath [autoRMSVoltage $mathData -1]
	}
		
		
	
}

proc autoRMSVoltage {data stepSize} {
	global t1Pos	\
		t2Pos		\
		xPlotStart	\
		xPlotWidth	\
		burstCount
	
	if {$t1Pos == $t2Pos} {
		return "?"
	}
	
	if {$t1Pos > $t2Pos} {
		set start $t2Pos
		set end $t1Pos
	} else {
		set start $t1Pos
		set end $t2Pos
	}
		
	#Determine the position of each cursor
	set startSample [expr {round(($start-$xPlotStart)*1.0/$xPlotWidth*$burstCount)}]
	set endSample [expr {round(($end-$xPlotStart)*1.0/$xPlotWidth*$burstCount)}]
	
	set sum 0
	for {set i $startSample} {$i <= $endSample} {incr i} {
		set datum [lindex $data $i]
		if {$stepSize != -1} {
			set datum [expr {(128.0-$datum)*$stepSize}]
		}
		set sum [expr {$sum+($datum*$datum)}]
	}
	set mean [expr {$sum/($endSample-$startSample)}]
	set rms [expr {sqrt($mean)}]
	set rms [formatAmplitude $rms]
	return $rms
	

}

#Auto Frequency Measurements
#---------------------------------
#This procedure is designed to automatically measure the frequency
#and the period of incoming data.  This procedure also computes
#the average of the incoming data.
proc autoFrequency {data stepSize} {
	global 	horizontalBox	\
			burstCount
		
	#Determine the number of samples
	set numSamples [llength $data]
	set numSamples [expr {$numSamples -1}]
	
	#Determine the average of the samples
	set average 0
	for {set i 0} {$i < $numSamples} {incr i} {
		set sampleValue [lindex $data $i]
		set average [expr {$average + $sampleValue}]
	}
	set average [expr {$average*1.0/$numSamples}]
	
	#Determine where the waveform crosses the average
	set crossings {}
	for {set i 1} {$i < $numSamples} {incr i} {
		if {[lindex $data $i] > $average && [lindex $data [expr {$i-1}]] <= $average} {
			lappend crossings $i
		} 
	}
	
	#Compute the frequency and period
	if {[llength $crossings] >= 2} {
		#Determine the average number of samples 
		#between  average crossings
		set betweenCrossings 0
		for {set i 1} {$i < [llength $crossings]} {incr i} {
			set betweenCrossings [expr $betweenCrossings + [expr [lindex $crossings $i] - [lindex $crossings [expr $i-1]]]]
		}
		set betweenCrossings [expr $betweenCrossings*1.0]
		set betweenCrossings [expr $betweenCrossings/[expr [llength $crossings]-1]]
			
		#Determine the amount of time represented by the
		#average  number of samples between average crossings
		set timeSweep [expr 10*$horizontalBox]
		set period [expr $timeSweep*$betweenCrossings/[expr $burstCount-1]]
		set frequency [expr 1/$period]
		#Format the results for display
		set frequency [formatFrequency $frequency]
		set period [formatPeriod $period]
	} else {
		set frequency "? Hz"
		set period "? s"
	}
	
	#Change the average sample value into a voltage
	if {$stepSize != -1} {
		set average [expr (128-$average)*$stepSize]
	}
	set average [formatAmplitude $average]
	
	#Set up a list of values to return
	set returnValue {}
	lappend returnValue $average
	lappend returnValue $frequency
	lappend returnValue $period
	return $returnValue
	
}

#Auto Amplitude
#----------------
#This procedure computes the maximum, minimum and peak to peak
#values for the incoming data
proc autoAmplitude {data stepSize} {

	#Find the maximum voltage value (which is the smallest
	#sample value) and the minimum voltage value (which is
	#the largest sample value).
	set length [llength $data]
	set length [expr {$length - 1}]
	set maximum -5000
	set minimum 5000
	for {set i 0} {$i < $length} {incr i} {
		if {[lindex $data $i] > $maximum} {
			set maximum [lindex $data $i]
		}
		if {[lindex $data $i] < $minimum} {
			set minimum [lindex $data $i]
		}
	}
	#Convert to real voltages and format
	if {$stepSize != -1} {
		set temp [expr {(128-$minimum)*$stepSize}]
		set minimum [expr {(128 - $maximum)*$stepSize}]
		set maximum $temp
	}
	set peakToPeak [expr {$maximum - $minimum}]
	set maximum [formatAmplitude $maximum]
	set minimum [formatAmplitude $minimum]
	set peakToPeak [formatAmplitude $peakToPeak]
	
	#Return the results as a list
	set returnValues {}
	lappend returnValues $maximum
	lappend returnValues $minimum
	lappend returnValues $peakToPeak
	return $returnValues
	
}

#Format Frequency
#-------------------
#This procedure takes a number and formats it into a string
#representing frequency with the proper units.  E.g. 1120 would
#be formated as "1.1 kHz"
proc formatFrequency {freq} {

	if {$freq > 1E6} {
		set freq [format "%3.1f" [expr $freq/1E6]]
		set freq "$freq MHz"
	} elseif { $freq > 1E3} {
		set freq [format "%3.1f" [expr $freq/1E3]]
		set freq "$freq kHz"
	} else {
		set freq [format "%3.1f" $freq]
		set freq "$freq Hz"
	}
	return $freq
}

#Format Period
#---------------
#This procedure takes a number representing a time period and
#formats it into a string with the proper units.
proc formatPeriod {period} {

	if {$period < 1E-6} {
		set period [format "%3.1f" [expr $period/1E-9]]
		set period "$period ns"
	} elseif {$period < 1E-3} {
		set period [format "%3.1f" [expr $period/1E-6]]
		set period "$period us"
	} elseif {$period < 1.0} {
		set period [format "%3.1f" [expr $period/1E-3]]
		set period "$period ms"
	} else {
		set period [format "%3.1f" $period]
		set period "$period s"
	}
	return $period
}

#Format Amplitude
#-------------------
#This procedure takes a number representing a voltage and
#formats it into a string with the proper units.
proc formatAmplitude {voltage} {

	if {[expr {abs($voltage)}] < 1.0} {
		set voltage [format "%3.0f" [expr $voltage/1E-3]]
		set voltage "$voltage mV"
	} else {
		set voltage [format "%2.2f" $voltage]
		set voltage "$voltage V"
	}

	return $voltage
}

