#File: calibration.tcl
#Syscomp USB Oscilloscope GUI
#Scope Vertical Calibration Procedures

#JG
#Copyright 2006 Syscomp Electronic Design
#www.syscompdesign.com

#This program is free software; you can redistribute it and/or
#modify it under the terms of the GNU General Public License as
#published by the Free Software Foundation; either version 2 of
#the License, or (at your option) any later verison.
#
#This program is distributed in the hope that it will be useful, but
#WITHOUT ANY WARRANTY; without even the implied warranty of
#MECHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See
#the GNU General Public License for more details.
#
#You should have received a copy of the GNU General Public License
#along with this program; if not, write to the Free Software
#Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301
#USA


#Calibration GUI
#------------------
#Thi procedure creates the calibration window and sets up all the widgets
#associate with calibrating the vertical scales.
proc calibration {} {
	global vertConstantsA	\
		vertConstantsB	\
		scaleA50mV	\
		scaleA100mV	\
		scaleA200mV	\
		scaleA500mV	\
		scaleA1V	\
		scaleA2V	\
		scaleA5V	\
		scaleB50mV	\
		scaleB100mV	\
		scaleB200mV	\
		scaleB500mV	\
		scaleB1V	\
		scaleB2V	\
		scaleB5V	\
		vertScaleAx1	\
		vertScaleAx10	\
		vertScaleBx1	\
		vertScaleBx10

	toplevel .calibrate
	wm title .calibrate "Scope Vertical Calibration"
	
	frame .calibrate.a -relief groove -borderwidth 3
	frame .calibrate.b -relief groove -borderwidth 3
	
	#Load Constants from Vertical Constant Arrays
	set scaleA50mV [expr {[lindex $vertConstantsA 0]/1.0}]
	set scaleA100mV [expr {[lindex $vertConstantsA 1]/0.5}]
	set scaleA200mV [expr {[lindex $vertConstantsA 2]/0.25}]
	set scaleA500mV [expr {[lindex $vertConstantsA 3]/0.1}]
	set scaleA1V [expr {[lindex $vertConstantsA 4]/0.05}]
	set scaleA2V [expr {[lindex $vertConstantsA 5]/0.025}]
	set scaleA5V [expr {[lindex $vertConstantsA 6]/0.01}]
	
	set scaleB50mV [expr {[lindex $vertConstantsB 0]/1.0}]
	set scaleB100mV [expr {[lindex $vertConstantsB 1]/0.5}]
	set scaleB200mV [expr {[lindex $vertConstantsB 2]/0.25}]
	set scaleB500mV [expr {[lindex $vertConstantsB 3]/0.1}]
	set scaleB1V [expr {[lindex $vertConstantsB 4]/0.05}]
	set scaleB2V [expr {[lindex $vertConstantsB 5]/0.025}]
	set scaleB5V [expr {[lindex $vertConstantsB 6]/0.01}]
	
	label .calibrate.chanA	\
		-text "Channel A"
	
	label .calibrate.a.label50mV	\
		-text "50mV/div"
	scale .calibrate.a.scale50mV	\
		-from 1.2	\
		-to 0.8	\
		-variable scaleA50mV	\
		-orient vertical	\
		-tickinterval 0	\
		-resolution 0.01	\
		-length 200		\
		-command {calibrateVertical a 0}
		
	label .calibrate.a.label100mV	\
		-text "100mV/div"
	scale .calibrate.a.scale100mV	\
		-from 1.2	\
		-to 0.8	\
		-variable scaleA100mV	\
		-orient vertical	\
		-tickinterval 0	\
		-resolution 0.01	\
		-length 200	\
		-command {calibrateVertical a 1}
	
	label .calibrate.a.label200mV	\
		-text "200mV/div"
	scale .calibrate.a.scale200mV	\
		-from 1.2	\
		-to 0.8	\
		-variable scaleA200mV	\
		-orient vertical	\
		-tickinterval 0	\
		-resolution 0.01	\
		-length 200	\
		-command {calibrateVertical a 2}
		
	label .calibrate.a.label500mV	\
		-text "500mV/div"
	scale .calibrate.a.scale500mV	\
		-from 1.2	\
		-to 0.8	\
		-variable scaleA500mV	\
		-orient vertical	\
		-tickinterval 0	\
		-resolution 0.01	\
		-length 200	\
		-command {calibrateVertical a 3}
		
	label .calibrate.a.label1V	\
		-text "1V/div"
	scale .calibrate.a.scale1V	\
		-from 1.2	\
		-to 0.8	\
		-variable scaleA1V	\
		-orient vertical	\
		-tickinterval 0	\
		-resolution 0.01	\
		-length 200	\
		-command {calibrateVertical a 4}
		
	label .calibrate.a.label2V	\
		-text "2V/div"	
	scale .calibrate.a.scale2V	\
		-from 1.2	\
		-to 0.8	\
		-variable scaleA2V	\
		-orient vertical	\
		-tickinterval 0 	\
		-resolution 0.01	\
		-length 200	\
		-command {calibrateVertical a 5}
		
	label .calibrate.a.label5V	\
		-text "5V/div"	
	scale .calibrate.a.scale5V	\
		-from 1.2	\
		-to 0.8	\
		-variable scaleA5V	\
		-orient vertical	\
		-tickinterval 0	\
		-resolution 0.01	\
		-length 200	\
		-command {calibrateVertical a 6}

	grid .calibrate.a.label50mV -row 0 -column 0
	grid .calibrate.a.scale50mV -row 1 -column 0
	grid .calibrate.a.label100mV -row 0 -column 1
	grid .calibrate.a.scale100mV -row 1 -column 1
	grid .calibrate.a.label200mV -row 0 -column 2
	grid .calibrate.a.scale200mV -row 1 -column 2
	grid .calibrate.a.label500mV -row 0 -column 3
	grid .calibrate.a.scale500mV -row 1 -column 3
	grid .calibrate.a.label1V -row 0 -column 4
	grid .calibrate.a.scale1V -row 1 -column 4
	grid .calibrate.a.label2V -row 0 -column 5
	grid .calibrate.a.scale2V -row 1 -column 5
	grid .calibrate.a.label5V -row 0 -column 6
	grid .calibrate.a.scale5V -row 1 -column 6
	
	label .calibrate.chanB	\
		-text "Channel B"
	
	#Channel B Calibration Controls
	label .calibrate.b.label50mV	\
		-text "50mV/div"
	scale .calibrate.b.scale50mV	\
		-from 1.2	\
		-to 0.8	\
		-variable scaleB50mV	\
		-orient vertical	\
		-tickinterval 0	\
		-resolution 0.01	\
		-length 200		\
		-command {calibrateVertical b 0}
		
	label .calibrate.b.label100mV	\
		-text "100mV/div"
	scale .calibrate.b.scale100mV	\
		-from 1.2	\
		-to 0.8	\
		-variable scaleB100mV	\
		-orient vertical	\
		-tickinterval 0	\
		-resolution 0.01	\
		-length 200	\
		-command {calibrateVertical b 1}
	
	label .calibrate.b.label200mV	\
		-text "200mV/div"
	scale .calibrate.b.scale200mV	\
		-from 1.2	\
		-to 0.8	\
		-variable scaleB200mV	\
		-orient vertical	\
		-tickinterval 0	\
		-resolution 0.01	\
		-length 200	\
		-command {calibrateVertical b 2}
		
	label .calibrate.b.label500mV	\
		-text "500mV/div"
	scale .calibrate.b.scale500mV	\
		-from 1.2	\
		-to 0.8	\
		-variable scaleB500mV	\
		-orient vertical	\
		-tickinterval 0	\
		-resolution 0.01	\
		-length 200	\
		-command {calibrateVertical b 3}
		
	label .calibrate.b.label1V	\
		-text "1V/div"
	scale .calibrate.b.scale1V	\
		-from 1.2	\
		-to 0.8	\
		-variable scaleB1V	\
		-orient vertical	\
		-tickinterval 0	\
		-resolution 0.01	\
		-length 200	\
		-command {calibrateVertical b 4}
		
	label .calibrate.b.label2V	\
		-text "2V/div"	
	scale .calibrate.b.scale2V	\
		-from 1.2	\
		-to 0.8	\
		-variable scaleB2V	\
		-orient vertical	\
		-tickinterval 0 	\
		-resolution 0.01	\
		-length 200	\
		-command {calibrateVertical b 5}
		
	label .calibrate.b.label5V	\
		-text "5V/div"	
	scale .calibrate.b.scale5V	\
		-from 1.2	\
		-to 0.8	\
		-variable scaleB5V	\
		-orient vertical	\
		-tickinterval 0	\
		-resolution 0.01	\
		-length 200	\
		-command {calibrateVertical b 6}	
	
	grid .calibrate.b.label50mV -row 0 -column 0
	grid .calibrate.b.scale50mV -row 1 -column 0
	grid .calibrate.b.label100mV -row 0 -column 1
	grid .calibrate.b.scale100mV -row 1 -column 1
	grid .calibrate.b.label200mV -row 0 -column 2
	grid .calibrate.b.scale200mV -row 1 -column 2
	grid .calibrate.b.label500mV -row 0 -column 3
	grid .calibrate.b.scale500mV -row 1 -column 3
	grid .calibrate.b.label1V -row 0 -column 4
	grid .calibrate.b.scale1V -row 1 -column 4
	grid .calibrate.b.label2V -row 0 -column 5
	grid .calibrate.b.scale2V -row 1 -column 5
	grid .calibrate.b.label5V -row 0 -column 6
	grid .calibrate.b.scale5V -row 1 -column 6
	
	label .calibrate.a.x1Label1	\
		-text "x1 Scaling Factor"
	label .calibrate.a.x1Label2	\
		-text "(50mV/div 100mV/div 200mV/div 500mV/div)"
		
	button .calibrate.restoreDefaults	\
		-text "Restore Defaults"	\
		-command {restoreDefaults}
	
	button .calibrate.saveCalibration	\
		-text "Save Calibration Values"	\
		-command {saveCalibration}
	
	grid .calibrate.chanA -row 0 -column 0 -sticky w
	grid .calibrate.a -row 1 -column 0
	grid .calibrate.chanB -row 2 -column 0 -sticky w
	grid .calibrate.b -row 3 -column 0 -sticky w
	grid .calibrate.restoreDefaults -row 4 -column 0
	grid .calibrate.saveCalibration -row 5 -column 0
}

#setVertical Redirect
#-----------------------
#This small procedure is used to call the main scope's setVertical command
#from the sliders in the calibration window.  The sliders automatically append
#their current value when calling commands.  This procedure strips off the
#extra argument from the slider and just calls setVertical
proc setVerticalRedirect {junkArgument} {
	setVertical
}

#Calibrate Vertical Scale
#---------------------------
#This procedure is the service routine called by each of the calibration sliders.
#It performs the necessary conversion calculations and updates the global arrays
#which hold the vertical constants for scaling the vertical axes.
proc calibrateVertical {chan vertIndex vertScale} {
	global vertConstantsA	\
		vertConstantsB
	
	switch $vertIndex {
		"0"	{ set vertValue [expr { 1.0*$vertScale}]
		} "1" { set vertValue [expr {0.5*$vertScale}]
		} "2" { set vertValue [expr {0.25*$vertScale}]
		} "3" { set vertValue [expr {0.1*$vertScale}]
		} "4" { set vertValue [expr {0.05*$vertScale}]
		} "5" { set vertValue [expr {0.025*$vertScale}]
		} "6" { set vertValue [expr {0.01*$vertScale}] }
	}
	
	switch $chan {
		"a" {
			set vertConstantsA [lreplace $vertConstantsA $vertIndex $vertIndex $vertValue]
		} "b" {
			set vertConstantsB [lreplace $vertConstantsB $vertIndex $vertIndex $vertValue]
		}
	}
	setVertical
	
}

#Restore Defaults
#-------------------
#This procedure restores all of the vertical scale settings to their defaults
proc restoreDefaults {} {
	global vertConstantsA	\
		vertConstantsB	\
		scaleA50mV	\
		scaleA100mV	\
		scaleA200mV	\
		scaleA500mV	\
		scaleA1V	\
		scaleA2V	\
		scaleA5V	\
		scaleB50mV	\
		scaleB100mV	\
		scaleB200mV	\
		scaleB500mV	\
		scaleB1V	\
		scaleB2V	\
		scaleB5V	\
		vertScaleAx1	\
		vertScaleAx10	\
		vertScaleBx1	\
		vertScaleBx10
	
	set vertConstantsA {1.0 0.5 0.25 0.1 0.05 0.025 0.01}
	set vertConstantsB {1.0 0.5 0.25 0.1 0.05 0.025 0.01}
	
	set vertScaleAx1 2.5
	set vertScaleAx10 2.5
	set vertScaleBx1 2.5
	set vertScaleBx10 2.5
	
	set scaleA50mV [expr {[lindex $vertConstantsA 0]/1.0}]
	set scaleA100mV [expr {[lindex $vertConstantsA 1]/0.5}]
	set scaleA200mV [expr {[lindex $vertConstantsA 2]/0.25}]
	set scaleA500mV [expr {[lindex $vertConstantsA 3]/0.1}]
	set scaleA1V [expr {[lindex $vertConstantsA 4]/0.05}]
	set scaleA2V [expr {[lindex $vertConstantsA 5]/0.025}]
	set scaleA5V [expr {[lindex $vertConstantsA 6]/0.01}]
	
	set scaleB50mV [expr {[lindex $vertConstantsB 0]/1.0}]
	set scaleB100mV [expr {[lindex $vertConstantsB 1]/0.5}]
	set scaleB200mV [expr {[lindex $vertConstantsB 2]/0.25}]
	set scaleB500mV [expr {[lindex $vertConstantsB 3]/0.1}]
	set scaleB1V [expr {[lindex $vertConstantsB 4]/0.05}]
	set scaleB2V [expr {[lindex $vertConstantsB 5]/0.025}]
	set scaleB5V [expr {[lindex $vertConstantsB 6]/0.01}]
	
	setVertical
}

#Save Calibration
#-------------------
#This procedure saves the current calibration values into a text configuration
#file that is used to configure the vertical scaling each time the program
#is started.
proc saveCalibration {} {
	global vertConstantsA	\
		vertConstantsB	\
		vertScaleAx1	\
		vertScaleAx10	\
		vertScaleBx1	\
		vertScaleBx10
		
	set configFile [open calibration.cfg w]
	puts $configFile "$vertConstantsA"
	puts $configFile "$vertConstantsB"
	puts $configFile $vertScaleAx1
	puts $configFile $vertScaleAx10
	puts $configFile $vertScaleBx1
	puts $configFile $vertScaleBx10
	close $configFile
}

#Attempt to read a vertical calibration file on startup
if [catch {open calibration.cfg r+} configFile] {
	puts "Unable to open custom calibration file, using defaults."
} else {
	gets $configFile line
	set vertConstantsA $line
	gets $configFile line
	set vertConstantsB  $line
	gets $configFile line
	set vertScaleAx1  $line
	gets $configFile line
	set vertScaleAx10  $line
	gets $configFile line
	set vertScaleBx1  $line
	gets $configFile line
	set vertScaleBx10  $line
	
	setVertical
	puts "Calibration constants loaded from file."
}