#File: dataRecorder.tcl
#Syscomp USB Oscilloscope GUI
#Data Recorder Toolbox

#JG
#Copyright 2007 Syscomp Electronic Design
#www.syscompdesign.com

#This program is free software; you can redistribute it and/or
#modify it under the terms of the GNU General Public License as
#published by the Free Software Foundation; either version 2 of
#the License, or (at your option) any later verison.
#
#This program is distributed in the hope that it will be useful, but
#WITHOUT ANY WARRANTY; without even the implied warranty of
#MECHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See
#the GNU General Public License for more details.
#
#You should have received a copy of the GNU General Public License
#along with this program; if not, write to the Free Software
#Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301
#USA

set recorderEnabled 0
set dataLogEnable 0
set numberLogs 0
set logPeriod 5
set recorderStatus "Idle"
set recordDir "."
set logNow 0

.menubar.tools.menu add command \
	-label "Data Recorder"	\
	-command {showRecorder}
.menubar.tools.menu add separator

proc showRecorder {} {
	global recorderEnabled
	
	if $recorderEnabled {wm deiconify .recorder; raise .recorder; return}
	
	toplevel .recorder
	wm title .recorder "Data Recorder Toolbox"
	wm resizable .recorder 0 0
	bind .recorder <Destroy> {set recorderEnabled 0}
	
	button .recorder.enableBox	\
		-text "Start Logging" \
		-command toggleRecording
		
	label .recorder.logsText	\
		-text "Log Count:"
		
	label .recorder.logsCount	\
		-textvariable numberLogs	\
		-width 10	\
		-relief sunken
		
	label .recorder.periodText 	\
		-text "Log Period (s):"
	
	entry .recorder.periodTime	\
		-textvariable logPeriod	\
		-width 10	\
		-justify center
		
	label .recorder.statusText	\
		-text "Recorder Status:"
		
	label .recorder.status	\
		-textvariable recorderStatus	\
		-width 20	\
		-relief sunken
		
	button .recorder.chooseDir	\
		-text "Choose Export Directory"	\
		-command {set recordDir [tk_chooseDirectory]}
	
	entry .recorder.currentDir	\
		-textvariable recordDir	\
		-width 30
		
	grid .recorder.enableBox -row 0 -column 0 -columnspan 2
	grid .recorder.logsText -row 1 -column 0 -sticky e
	grid .recorder.logsCount -row 1 -column 1 -sticky w
	grid .recorder.periodText -row 2 -column 0 -sticky e
	grid .recorder.periodTime -row 2 -column 1 -sticky w
	grid .recorder.statusText -row 3 -column 0 -sticky e
	grid .recorder.status -row 3 -column 1 -sticky w
	grid .recorder.chooseDir -row 4 -column 0 -columnspan 2
	grid .recorder.currentDir -row 5 -column 0 -columnspan 2
	
	
	set recorderEnabled 1
}

proc toggleRecording {} {
	global dataLogEnable	\
		logPeriod	\
		logNow	\
		numberLogs
	
	if $dataLogEnable {
		.recorder.enableBox configure -text "Start Logging"
		set dataLogEnable 0
		.recorder.periodTime configure -state normal
	} else {
	
		#Check to make sure that timer interval is an integer
		if {![string is integer -strict $logPeriod]} {
			tk_messageBox	\
				-message "Invalid Log Period\nPeriod must be integer >= 1"	\
				-type ok
				return
		}
		if {$logPeriod < 1} {
			tk_messageBox	\
				-message "Invalid Log Period\nPeriod must be integer >= 1"	\
				-type ok
				return
		}
	
		.recorder.enableBox configure -text "Stop Logging"
		set dataLogEnable 1
		set numberLogs 0
		.recorder.periodTime configure -state disabled
		set logNow 1
	}
	
}

proc dataRecord {} {
	global logNow	\
		logPeriod	\
		dataLogEnable	\
		exportData	\
		recordDir	\
		numberLogs	\
		recorderEnabled
	
	set logNow 0
	
	if $dataLogEnable {
	
		#Get date for file (i.e. Mon-Day-Year-HH-MM-SS)
		set now [clock seconds]
		set month [clock format $now -format %b]
		set day [clock format $now -format %d]
		set year [clock format $now -format %Y]
		set hour [clock format $now -format %k]
		set minute [clock format $now -format %M]
		set seconds [clock format $now -format %S]
		
		set filename "$month-$day-$year-$hour-$minute-$seconds.csv"
		
		set fullPath "$recordDir/$filename"
		
		#Copied from Export.tcl:
		set xData [lindex $exportData 0]
		set aData [lindex $exportData 1]
		set bData [lindex $exportData 2]
		set horizontalBox [lindex $exportData 3]
		set stepA [lindex $exportData 4]
		set stepB [lindex $exportData 5]
		set dataLength [llength $xData]
		set timeStep [expr {$horizontalBox*10/$dataLength}]
		
		#if{[catch {open $fullPath w} fileId]} {
		#	tk_messageBox	\
		#		-message "Unable to Open File for Writing"	\
		#		-type ok
		#	return
		#}
		set fileId [open $fullPath w]
		puts $fileId {"Time [s],Channel A [V],Channel B [V]"}
		set i 0
		foreach xValue $xData aValue $aData bValue $bData {
			puts -nonewline $fileId [expr {$i*$timeStep}]
			puts -nonewline $fileId ","
			puts -nonewline $fileId [expr {(128-$aValue)*$stepA}]
			puts -nonewline $fileId ","
			puts $fileId [expr {(128-$bValue)*$stepB}]
			incr i
			#puts $fileId "$xValue,$aValue,$bValue"
		}
		close $fileId
		
		incr numberLogs
		
		if $recorderEnabled {
			set msPeriod [expr $logPeriod*1000]
			after $msPeriod {set logNow 1}
		}
	}
	
}
