#File: global.tcl
#Syscomp USB Oscilloscope GUI
#Global Variable File

#JG
#Copyright 2007 Syscomp Electronic Design
#www.syscompdesign.com

#This program is free software; you can redistribute it and/or
#modify it under the terms of the GNU General Public License as
#published by the Free Software Foundation; either version 2 of
#the License, or (at your option) any later verison.
#
#This program is distributed in the hope that it will be useful, but
#WITHOUT ANY WARRANTY; without even the implied warranty of
#MECHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See
#the GNU General Public License for more details.
#
#You should have received a copy of the GNU General Public License
#along with this program; if not, write to the Free Software
#Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301
#USA

#======================================
# Global Variables
# ALL global variables should (eventually) be listed here
#======================================


set companyString "Syscomp Electronic Design Ltd"
set deviceString "USB Digital Storage Oscilloscope"
set firmwareIdent "Unknown"

#Display Related
set title "Syscomp Digital Storage Oscilloscope"
set triggerStateTag "triggerState"
set manuallyTriggered 0
set serialStatus "Disconnected"
set otherPort {}
set connectMessage "Unable to connect to device.\n Examine connection settings?"

#Screen Geometry
set xPlotWidth 400
set yPlotWidth 400
set cursorBorder 15
set xPlotStart $cursorBorder
set xPlotEnd [expr $xPlotWidth + $cursorBorder]
set yPlotStart $cursorBorder
set yPlotEnd [expr $yPlotWidth + $cursorBorder]
set xMid [expr ($xPlotEnd - $xPlotStart)/2 + $cursorBorder]
set xDiv10 [expr ($xPlotEnd - $xPlotStart)/10 ]
set yMid [expr ($yPlotEnd - $yPlotStart)/2 + $cursorBorder]
set yDiv10 [expr ($yPlotEnd - $yPlotStart)/10]
set yHashLength [expr $yDiv10/8]
set xHashLength [expr $xDiv10/8]
set yMinorHash [expr $yDiv10/20]
set xMinorHash [expr $xDiv10/20]
set yStart $yMid

#Initial Cursor Positions
set chAGndPos [expr $yMid]
set chBGndPos [expr $yMid]
set trigPos [expr $yMid]
set triggerLevel 128
set timePos [expr $xMid]
set t1Pos [expr $xPlotStart]
set t2Pos [expr $xPlotStart + $xPlotWidth/10]
set timeCursorsEnable 1
set va1Pos [expr $yMid-2*$yDiv10]
set va2Pos [expr $yMid-$yDiv10]
set vb1Pos [expr $yMid+$yDiv10]
set vb2Pos [expr $yMid+2*$yDiv10]
set voltageCursorsEnable 1

#Plotting Waveforms
set waveTagA "waveformA"
set waveColorA red
set waveOffsetA 0
set waveTagB "waveformB"
set waveColorB blue
set waveOffsetB 0
set autoUpdate 0
set dataA [list]
set dataB [list]

#Hardware Specific 
set preampGains {0.04 0.08 0.16 0.32 0.4 0.8 1.6 3.2}
set adStepSize 0.00521
set intExt 0
#Baud Rate for Initial Start Up
set baudSlow 38400
set baudRate $baudSlow
set scopeInitialized 0
set autoTriggerHandle 0

#Widget Variables
set triggerModes { "Auto" "Normal" "Single-Shot" "Hi-Res"}
set triggerSlopes { "Plus" "Minus" }
set triggerSlope "Plus"
set triggerSources { "A" "B"}
set verticalValues {"50mV/div" \
	"100mV/div" "200mV/div" "500mV/div" "1V/div" "2V/div" "5V/div"}
set verticalIndexA 4
set verticalA [lindex $verticalValues $verticalIndexA]
set verticalIndexB 4
set verticalB [lindex $verticalValues $verticalIndexB]
set vertConstantsA {1.0 0.5 0.25 0.1 0.05 0.025 0.01}
set vertConstantsB {1.0 0.5 0.25 0.1 0.05 0.025 0.01}
set vertScaleAx1 2.5
set vertScaleAx10 2.5
set vertScaleBx1 2.5
set vertScaleBx10 2.5
set timebaseValues {"50ns/div" "100ns/div" "200ns/div" "500ns/div" \
	"1us/div" "2us/div" "5us/div" "10us/div" "20us/div" "50us/div"	 \
	"100us/div" "200us/div" "500us/div" "1ms/div" "2ms/div" "5ms/div" \
	"10ms/div" "20ms/div" "50ms/div" "100ms/div" "200ms/div"}
set timebaseIndex 13
set timebaseSetting [lindex $timebaseValues $timebaseIndex]
set triggerFilter 0

#GUI Graphics
set zoomInImage [image create photo -file "$images/MagIn.gif"]
set zoomOutImage [image create photo -file "$images/MagOut.gif"]
set zoomOutImage [image create photo -file "$images/MagOut.gif"]
set timebaseImage [image create photo -file "$images/1ms.gif"]
set verticalAImage [image create photo -file "$images/a1V.gif"]
set verticalBImage [image create photo -file "$images/b1V.gif"]
set channelAHeader [image create photo -file "$images/ChannelA.gif"]
set channelBHeader [image create photo -file "$images/ChannelB.gif"]
set triggerDisplay [image create photo -file "$images/aRising.gif"]
set titleImage [image create photo -file "$images/Title.gif"]
set autoFrequency [image create photo -file "$images/AutoFrequency.gif"]
set autoAverage [image create photo -file "$images/AutoAverage.gif"]
set autoMax [image create photo -file "$images/AutoMax.gif"]
set autoMin [image create photo -file "$images/AutoMin.gif"]
set autoPkPk [image create photo -file "$images/AutoPkPk.gif"]
set autoRMS [image create photo -file "$images/AutoRMS.gif"]
set timebaseImages [image create photo -file "$images/50ns.gif"]
lappend timebaseImages [image create photo -file "$images/100ns.gif"]
lappend timebaseImages [image create photo -file "$images/200ns.gif"]
lappend timebaseImages [image create photo -file "$images/500ns.gif"]
lappend timebaseImages [image create photo -file "$images/1us.gif"]
lappend timebaseImages [image create photo -file "$images/2us.gif"]
lappend timebaseImages [image create photo -file "$images/5us.gif"]
lappend timebaseImages [image create photo -file "$images/10us.gif"]
lappend timebaseImages [image create photo -file "$images/20us.gif"]
lappend timebaseImages [image create photo -file "$images/50us.gif"]
lappend timebaseImages [image create photo -file "$images/100us.gif"]
lappend timebaseImages [image create photo -file "$images/200us.gif"]
lappend timebaseImages [image create photo -file "$images/500us.gif"]
lappend timebaseImages [image create photo -file "$images/1ms.gif"]
lappend timebaseImages [image create photo -file "$images/2ms.gif"]
lappend timebaseImages [image create photo -file "$images/5ms.gif"]
lappend timebaseImages [image create photo -file "$images/10ms.gif"]
lappend timebaseImages [image create photo -file "$images/20ms.gif"]
lappend timebaseImages [image create photo -file "$images/50ms.gif"]
lappend timebaseImages [image create photo -file "$images/100ms.gif"]
lappend timebaseImages [image create photo -file "$images/200ms.gif"]
set triggerImages [image create photo -file "$images/aRising.gif"]
lappend triggerImages [image create photo -file "$images/aFalling.gif"]
lappend triggerImages [image create photo -file "$images/bRising.gif"]
lappend triggerImages [image create photo -file "$images/bFalling.gif"]
set aImages [image create photo -file "$images/a50mV.gif"]
lappend aImages [image create photo -file "$images/a100mV.gif"]
lappend aImages [image create photo -file "$images/a200mV.gif"]
lappend aImages [image create photo -file "$images/a500mV.gif"]
lappend aImages [image create photo -file "$images/a1V.gif"]
lappend aImages [image create photo -file "$images/a2V.gif"]
lappend aImages [image create photo -file "$images/a5V.gif"]
set bImages [image create photo -file "$images/b50mV.gif"]
lappend bImages [image create photo -file "$images/b100mV.gif"]
lappend bImages [image create photo -file "$images/b200mV.gif"]
lappend bImages [image create photo -file "$images/b500mV.gif"]
lappend bImages [image create photo -file "$images/b1V.gif"]
lappend bImages [image create photo -file "$images/b2V.gif"]
lappend bImages [image create photo -file "$images/b5V.gif"]

#Add-on Variables
set histoEnabled 0
set measurementsEnable 1

#Sampe Rate and Hi-Res Captures
set sampleRates {"20MS/s" "10MS/s" "5MS/s" "2.5MS/s" "1.25MS/s" \
	"625kS/s" "312kS/s" "156kS/s" "78kS/s" "39kS/s" "19kS/s" "9.7kS/s" \
	"4.9kS/s"}
set sampleIntervals {"50ns" "100ns" "200ns" "400ns" "800ns" "1.6us" \
	"3.2us" "6.4us" "12.8us" "25.6us" "51.2us" "102.4us" "204.8us" }
set hrEnable 1