#File: histogram.tcl
#Syscomp USB Oscilloscope GUI
#Histogram Add-on for Oscilloscope

#JG
#Ver 1.0 January 2006
#Copyright 2006 Syscomp Electronic Design
#www.syscompdesign.com

#This program is free software; you can redistribute it and/or
#modify it under the terms of the GNU General Public License as
#published by the Free Software Foundation; either version 2 of
#the License, or (at your option) any later verison.
#
#This program is distributed in the hope that it will be useful, but
#WITHOUT ANY WARRANTY; without even the implied warranty of
#MECHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See
#the GNU General Public License for more details.
#
#You should have received a copy of the GNU General Public License
#along with this program; if not, write to the Free Software
#Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301
#USA

set histoHeight 240

set histoYAxis 100
set histoPlotWidth 256
set histoPlotHeight 200
set histoWidth [expr $histoYAxis + $histoPlotWidth + 10]

set histoPlotTop 10
set histoPlotBottom [expr $histoPlotTop + $histoPlotHeight]
set histoCursorPosition [expr $histoYAxis + 128]
set cursorSamplePosition 128

#Create Histogram
#------------------
#When invoked, this procedure creates a new window for the
#histogram display.
proc createHistogram {} {
	global histoHeight	\
		histoWidth	\
		histoCursorPosition	\
		histoPlotTop	\
		histoPlotBottom	\
		histoYAxis	\
		histoPlotHeight	\
		histoPlotWidth	\
		histoEnabled	\
		enableHistoA	\
		enableHistoB

	if {[winfo exists .histo]} {
		raise .histo
		return
	}

	toplevel .histo
	wm title .histo "Histogram"
	wm iconname .histo "Histogram"
	wm resizable .histo 0 0
	
	bind .histo <Destroy> {set histoEnabled 0}
	
	canvas .histo.display	\
		-width $histoWidth	\
		-height $histoHeight	\
		-background white	\
		-borderwidth 2
		
	grid .histo.display -row 0 -column 0
	
	#Draw X Axis and Labels
	.histo.display create line	\
		$histoYAxis $histoPlotBottom	\
		[expr $histoYAxis + $histoPlotWidth] $histoPlotBottom
	.histo.display create text	\
		[expr $histoYAxis + ($histoPlotWidth/2)] [expr $histoPlotBottom + 20]	\
		-text "Sample Value"
	.histo.display create text	\
		$histoYAxis [expr $histoPlotBottom + 10]	\
		-text "0"
	.histo.display create text	\
		[expr $histoYAxis + ($histoPlotWidth/2)] [expr $histoPlotBottom + 10]	\
		-text "128"
	.histo.display create text	\
		[expr $histoYAxis + $histoPlotWidth] [expr $histoPlotBottom + 10]	\
		-text "255"
		
	#Draw Y Axis and Labels
	.histo.display create line	\
		$histoYAxis $histoPlotBottom	\
		$histoYAxis $histoPlotTop
	.histo.display create text	\
		[expr $histoYAxis/2] [expr ($histoPlotBottom - $histoPlotTop)/2]	\
		-text "% of\n total\n samples"
	.histo.display create text	\
		[expr $histoYAxis - 10] $histoPlotBottom	\
		-text 0
	.histo.display create text	\
		[expr $histoYAxis - 10] [expr ($histoPlotBottom - $histoPlotTop)/2]	\
		-text 50
	.histo.display create text	\
		[expr $histoYAxis - 10] $histoPlotTop	\
		-text 100
	
	#Measurement Cursor
	.histo.display create line	\
		[expr $histoYAxis + 128] 210	\
		[expr $histoYAxis + 128] 10	\
		-tag histoCursor		\
		-fill green
	.histo.display create oval	\
		[expr $histoYAxis + 125] 13	\
		[expr $histoYAxis + 131] 7	\
		-tag histoCursor	\
		-fill green
	
	.histo.display bind histoCursor <Button-1> {markHisto %x}
	.histo.display bind histoCursor <B1-Motion> {moveHisto %x}
		
	frame .histo.tools

	checkbutton .histo.tools.enableA	\
		-text "Enable A"	\
		-variable enableHistoA
		
	checkbutton .histo.tools.enableB	\
		-text "Enable B"	\
		-variable enableHistoB

	label .histo.tools.labelCursorPositon	\
		-text "Cursor Position"

	label .histo.tools.cursorPosition	\
		-textvariable cursorSamplePosition	\
		-width 12	\
		-relief sunken
		
	label .histo.tools.labelChASamples	\
		-text "Channel A"
		
	label .histo.tools.chASamples	\
		-textvariable chASamples	\
		-relief sunken	\
		-width 8
		
	label .histo.tools.labelChBSamples	\
		-text "Channel B"
		
	label .histo.tools.chBSamples	\
		-textvariable chBSamples	\
		-relief sunken	\
		-width 8
	
		
	grid .histo.tools.enableA -row 0 -column 0 -columnspan 2
	grid .histo.tools.enableB -row 1 -column 0 -columnspan 2
	grid .histo.tools.labelCursorPositon -row 2 -column 0
	grid .histo.tools.cursorPosition -row 2 -column 1
	grid .histo.tools.labelChASamples -row 3 -column 0
	grid .histo.tools.chASamples -row 3 -column 1
	grid .histo.tools.labelChBSamples -row 4 -column 0
	grid .histo.tools.chBSamples -row 4 -column 1
	
	grid .histo.tools -row 0 -column 1
	
	set enableHistoA 1
	set enableHistoB 1

	set histoEnabled 1	
		
}

proc markHisto {xPos} {
	global histoCursorPosition	\
		histoXStart
	
	set dy 0
	
	set snap [expr $xPos - $histoCursorPosition]
	set histoCursorPosition $xPos
	.histo.display move histoCursor $snap $dy
	
	set histoXStart $xPos

}


proc moveHisto {xPos} {
	global histoCursorPosition	\
		histoXStart	\
		histoYAxis	\
		histoPlotWidth	\
		cursorSamplePosition
		
	if {$xPos < $histoYAxis} {set xPos $histoYAxis}
	if {$xPos > [expr $histoYAxis + $histoPlotWidth]} {
		set xPos [expr $histoYAxis + $histoPlotWidth]
	}
	
	set dy 0
	set dx [expr $xPos - $histoXStart]
	set histoXStart $xPos
	set histoCursorPosition $xPos
	.histo.display move histoCursor $dx $dy

	set cursorSamplePosition [expr $histoCursorPosition - $histoYAxis]

}

proc updateHisto {} {
	global	histoPlotBottom	\
			histoYAxis	\
			histoPlotHeight	\
			enableHistoA	\
			enableHistoB	\
			chASamples	\
			chBSamples	\
			cursorSamplePosition	\
			exportData

	set dataARaw [lindex $exportData 1]
	set dataBRaw [lindex $exportData 2]

	#Remove the old bars
	.histo.display delete histoBarsA
	.histo.display delete histoBarsB

	#Create an array for counting the number of samples
	set histoCount {}
	for {set i 0} {$i <256} {incr i} {
		lappend histoCount 0
	}
	
	#Histogram Display for Data from Channel A
	if {$enableHistoA} {
		#Remove the empty list at the end of the list
		#set dataARaw [lrange $dataARaw 0 end-1]
	
		#Count the number of samples and save them
		#in the histoCount array
		foreach dataValue $dataARaw {
			set temp [lindex $histoCount $dataValue]
			incr temp
			set histoCount [lreplace $histoCount $dataValue $dataValue $temp]
		}
	
		set numberSamples [llength $dataARaw]
		set i 0
		
		foreach histoValue $histoCount {
			set xPosition [expr {$histoYAxis+$i}]
			set yLength [expr {($histoValue*1.0)/$numberSamples*$histoPlotHeight}]
			.histo.display create line	\
				$xPosition $histoPlotBottom	\
				$xPosition [expr {$histoPlotBottom-$yLength}]	\
				-fill red	\
				-tag histoBarsA
			incr i
		}
		
		set chASamples [lindex $histoCount $cursorSamplePosition]
	} else {
		set chASamples ""
	}
	
	#Create an array for counting the number of samples
	set histoCount {}
	for {set i 0} {$i <256} {incr i} {
		lappend histoCount 0
	}
	
	if {$enableHistoB} {
		#Remove the empty list item at the end of the list
		#set dataBRaw [lrange $dataBRaw 0 end-1]
	
		#Count the number of samples and record the
		#values in histoCount
		foreach dataValue $dataBRaw {
			set temp [lindex $histoCount $dataValue]
			incr temp
			set histoCount [lreplace $histoCount $dataValue $dataValue $temp]
		}
	
		set numberSamples [llength $dataBRaw]
		set i 0
		foreach histoValue $histoCount {
			set xPosition [expr {$histoYAxis+$i}]
			set yLength [expr {($histoValue*1.0)/$numberSamples*$histoPlotHeight}]
			.histo.display create line	\
				$xPosition $histoPlotBottom	\
				$xPosition [expr $histoPlotBottom-$yLength]	\
				-fill blue	\
				-tag histoBarsB
			incr i
		}
		
		set chBSamples [lindex $histoCount $cursorSamplePosition]
	} else {
		set chBSamples ""
	}
}

#Add an entry to the "tools" menu for the histogram
.menubar.tools.menu add command	\
	-label "Histogram"	\
	-command createHistogram
.menubar.tools.menu add separator