#File: main.tcl
#Syscomp USB Oscilloscope GUI
#Main File

#JG
set softwareVersion "1.11"
#Copyright 2007 Syscomp Electronic Design
#www.syscompdesign.com

#This program is free software; you can redistribute it and/or
#modify it under the terms of the GNU General Public License as
#published by the Free Software Foundation; either version 2 of
#the License, or (at your option) any later verison.
#
#This program is distributed in the hope that it will be useful, but
#WITHOUT ANY WARRANTY; without even the implied warranty of
#MECHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See
#the GNU General Public License for more details.
#
#You should have received a copy of the GNU General Public License
#along with this program; if not, write to the Free Software
#Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301
#USA

#======================================
# Global Variables
#======================================

#Determine the operating system
set osType $tcl_platform(platform)
if {$osType == "unix"} {
	if {[exec uname] == "Darwin"} {set osType "Darwin"}
}
set include "./"

#Img package required for screen captures
#lappend auto_path "$include/Img1.3"
if {$osType=="windows"} {
	lappend auto_path "$include/Img13Win"
} else {
	lappend auto_path "$include/Img13Lin"
}
package require Img

#BWidget package used for comboboxes
if {$osType=="windows"} {
	lappend auto_path "$include/Bwidget18Win"
} else {
	lappend auto_path "$include/Bwidget18Lin"
}
package require BWidget

set images "$include/Images"

source "$include/global.tcl"
source "$include/cursors.tcl"
source "$include/usb_serial.tcl"
source "$include/capture.tcl"

#======================================
# Procedures
#======================================

#Initialize Scope
#----------------
#This procedure initializes the scope for its first capture.  All GUI
#settings are sent to the hardware: vertical settings, timebase
#settings, trigger settings, etc.
proc initializeScope {} {
		global burstCount \
			newBurstCount \
			burstIncrement \
			newBurstIncrement \
			timePos	\
			scopeInitialized

		#Initialize the Vertical Settings
		setVertical
		
		#Initialize the Timebase
		adjustSettings		
		set burstCount $newBurstCount
		set highByte [expr {$burstCount/256}]
		set highByte [expr {floor($highByte)}]
		set highByte [expr {round($highByte)}]
		set lowByte [expr {$burstCount-($highByte*256)}]
		sendCommand "c$lowByte"
		sendCommand "v$highByte"
		set burstIncrement $newBurstIncrement
		sendCommand "h0"
		sendCommand "l$burstIncrement"
		adjustTimeOffset $timePos
		createXData
		set updateTimebaseFlag 0
		
		drawXCursor
		
		set scopeInitialized 1
}


#Adjust Vertical Settings
#-------------------------
#This procedure is used to adjust the vertical settings.  The
#vertical settings are mapped to control values for the hardware
#preamp and the command to adjust the preamp settings are
#sent to the hardware.
proc setVertical {} {
	global verticalA	\
		verticalB	\
		preampA	\
		preampB	\
		stepA	\
		stepB		\
		verticalBoxA	\
		verticalBoxB	\
		adStepSize	\
		trigPos	\
		yStart	\
		verticalAImage	\
		verticalBImage	\
		images	\
		vertConstantsA	\
		vertConstantsB	\
		vertScaleAx1	\
		vertScaleAx10	\
		vertScaleBx1	\
		vertScaleBx10	\
		aImages		\
		bImages

	#Here we create the control integer for the preamp
	set preampValue 0

	switch $verticalA {
		"50mV/div" {
			#set preampA 1.0
			set preampA [lindex $vertConstantsA 0]
			set preampValue [expr {$preampValue + 1}]
			set verticalBoxA .05
			set verticalAImage [lindex $aImages 0]
			set vertScaleA $vertScaleAx1
		} "100mV/div" {
			#set preampA 0.5
			set preampA [lindex $vertConstantsA 1]
			set preampValue [expr {$preampValue + 9}]
			set verticalBoxA 0.1
			set verticalAImage [lindex $aImages 1]
			set vertScaleA $vertScaleAx1
		} "200mV/div" {
			#set preampA 0.25
			set preampA [lindex $vertConstantsA 2]
			set preampValue [expr {$preampValue + 5}]
			set verticalBoxA 0.2
			set verticalAImage [lindex $aImages 2]
			set vertScaleA $vertScaleAx1
		} "500mV/div" {
			#set preampA 0.1
			set preampA [lindex $vertConstantsA 3]
			set preampValue [expr {$preampValue + 13}]
			set verticalBoxA 0.5
			set verticalAImage [lindex $aImages 3]
			set vertScaleA $vertScaleAx1
		} "1V/div" {
			#set preampA 0.05
			set preampA [lindex $vertConstantsA 4]
			set preampValue [expr {$preampValue + 8}]
			set verticalBoxA 1.0
			set verticalAImage [lindex $aImages 4]
			set vertScaleA $vertScaleAx10
		} "2V/div" {
			#set preampA 0.025
			set preampA [lindex $vertConstantsA 5]
			set preampValue [expr {$preampValue + 4}]
			set verticalBoxA 2.0
			set verticalAImage [lindex $aImages 5]
			set vertScaleA $vertScaleAx10
		} "5V/div" {
			#set preampA 0.01
			set preampA [lindex $vertConstantsA 6]
			set preampValue [expr {$preampValue + 12}]
			set verticalBoxA 5.0
			set verticalAImage [lindex $aImages 6]
			set vertScaleA $vertScaleAx10
		}
	}
	.vertical.cha.sensitivity configure -image $verticalAImage
	
	switch $verticalB {
		"50mV/div" {
			#set preampB 1.0
			set preampB [lindex $vertConstantsB 0]
			set preampValue [expr {$preampValue + 16}]
			set verticalBoxB .05
			set verticalBImage [lindex $bImages 0]
			set vertScaleB $vertScaleBx1
		} "100mV/div" {
			#set preampB 0.5
			set preampB [lindex $vertConstantsB 1]
			set preampValue [expr {$preampValue + 144}]
			set verticalBoxB 0.1
			set verticalBImage [lindex $bImages 1]
			set vertScaleB $vertScaleBx1
		} "200mV/div" {
			#set preampB 0.25
			set preampB [lindex $vertConstantsB 2]
			set preampValue [expr {$preampValue + 80}]
			set verticalBoxB 0.2
			set verticalBImage [lindex $bImages 2]
			set vertScaleB $vertScaleBx1
		} "500mV/div" {
			#set preampB 0.1
			set preampB [lindex $vertConstantsB 3]
			set preampValue [expr {$preampValue + 208}]
			set verticalBoxB 0.5
			set verticalBImage [lindex $bImages 3]
			set vertScaleB $vertScaleBx1
		} "1V/div" {
			#set preampB 0.05
			set preampB [lindex $vertConstantsB 4]
			set preampValue [expr {$preampValue + 128}]
			set verticalBoxB 1.0
			set verticalBImage [lindex $bImages 4]
			set vertScaleB $vertScaleBx10
		} "2V/div" {
			#set preampB 0.025
			set preampB [lindex $vertConstantsB 5]
			set preampValue [expr {$preampValue + 64}]
			set verticalBoxB 2.0
			set verticalBImage [lindex $bImages 5]
			set vertScaleB $vertScaleBx10
		} "5V/div" {
			#set preampB 0.01
			set preampB [lindex $vertConstantsB 6]
			set preampValue [expr {$preampValue + 192}]
			set verticalBoxB 5.0
			set verticalBImage [lindex $bImages 6]
			set vertScaleB $vertScaleBx10
		}
		
	}
	.vertical.chb.sensitivity configure -image $verticalBImage
	
	#Send the preamp control command to the hardware
	sendCommand "p$preampValue"
	
	#Calculate the voltage step size for each sample
	set stepA [expr {(1/$preampA)*$adStepSize/$vertScaleA}]
	set stepB [expr {(1/$preampB)*$adStepSize/$vertScaleB}]
	
	#Adjust the trigger level accordingly.  We set the yStart
	#coordinate to the current position of the trigger cursor to 
	#simulate a click and ensure that the trigger level is 
	#referenced to the correct screen position.
	set yStart $trigPos
	moveTrigger  $trigPos
	
	#Recalculate cursor positions at new vertical setting
	calculateCursors
	
}

#Acquire Waveform
#-------------------
#This process requests a data capture from the scope.
#The "manualTrigger" command is registered to occur after
#a short period.  In auto trigger mode this will cause the scope
#to trigger.  In normal mode, this will update the display to 
#indicate that the scope is not triggered.
proc acquireWaveform {} {
	global autoTriggerHandle	\
		dataA	\
		dataB	\
		dataARaw	\
		dataBRaw	\
		triggerMode	\
		dataCount	\
		unsigned
	
	#Create new, empty lists for the incoming data
	set dataA {}
	set dataB {}
	set dataARaw {}
	set dataBRaw {}
	
	#Clear data buffers
	set dataCount 0
	set unsigned {}
	
	#Request a capture
	sendCommand "g"

	#Register the manual trigger command
	if { ($triggerMode != "Single-Shot") && ($triggerMode != "Hi-Res") } {
		set autoTriggerHandle [after 1000 {manualTrigger}]
	}
}


#Manual Trigger
#---------------
#The purpose of this procedure is two-fold.  First, it will manually
#trigger the scope to perform a capture if the scope is set to
#autotrigger after a preset time.  Secondly, this procedure is
#used to manage the triggered/not triggered prompt on the
#display.
proc manualTrigger {} {
	global intExt			\
		manuallyTriggered	\
		triggerMode		\
		triggerStateTag		\
		xMid				\
		yPlotEnd			\
		cursorBorder
	
	#Manually trigger the scope if we are in auto trigger mode
	if { $triggerMode == "Auto" } {
		set intExt 1
		adjustSettings
		sendCommand "m"
		set intExt 0
		adjustSettings
	}
	
	#Update the display to indicate that a manual trigger was
	#necessary
	set triggerState "Not Triggered"
	set manuallyTriggered 1
	.display delete $triggerStateTag
	.display create text	\
		[expr {$xMid}] [expr {$yPlotEnd+$cursorBorder/2}]	\
		-fill black	\
		-tag $triggerStateTag	\
		-text $triggerState
}

#Select Channel
#---------------
#This procedure manages the enabling and disabling of the input
#channels on the display. 
proc selectChannel { } {
	global enableA	\
		enableB	\
		waveTagA	\
		waveTagB	\
		triggerMode
	
	#Delete the waveforms if they are disabled.
	if { $enableA == 0 } {
		.display delete $waveTagA
	}
	if { $enableB == 0} {
		.display delete $waveTagB
	}
	
	acquireWaveform
	
}

#Adjust Settings
#----------------
#This procedure is used to adjust the timebase and trigger settings
#of the hardware.  These settings are not applied until the end of
#the next capture cycle (refer to the updateOscilloscope procedure)
#to ensure that the GUI and hardware stay in sync.
proc adjustSettings {} {
	global timebaseSetting	\
		triggerSource	\
		triggerSlope	\
		intExt	\
		newBurstCount	\
		newBurstIncrement	\
		updateTimebaseFlag	\
		horizontalBox	\
		timeCursorsEnable	\
		timebaseImage	\
		triggerDisplay	\
		images	\
		triggerMode	\
		timebaseImages	\
		triggerImages	\
		sampleRate		\
		triggerFilter
		
	if {$triggerMode == "Hi-Res"} {
		adjustSampleRate
		return
	}
		
	#Initial Value for register (Internal trigger setting)
	if {$intExt} {
		#External Trigger
		set registerValue 0
	} else {
		#Internal Trigger
		set registerValue 16
	}
	
	#Trigger Source
	if { $triggerSource ==  "B" } {
		set registerValue [expr {$registerValue + 64}]
	}
	
	#Trigger Slope
	if { $triggerSlope == "Minus" } {
		set registerValue [expr {$registerValue + 32}]
	}
	
	#Trigger Filtering
	if {$triggerFilter} {
		#Disable trigger filtering
		set registerValue [expr {$registerValue + 128}]
	}
	
	#Trigger Display
	if {$triggerSource == "A" } {
		if {$triggerSlope == "Plus"} {
			set triggerDisplay [lindex $triggerImages 0]
		} else {
			set triggerDisplay [lindex $triggerImages 1]
		}
	} else {
		if {$triggerSlope == "Plus"} {
			set triggerDisplay [lindex $triggerImages 2]
		} else {
			set triggerDisplay [lindex $triggerImages 3]
		}
	}
	.rhs.trigger.display configure -image $triggerDisplay

	#Timebase Setting
	switch $timebaseSetting {
		"50ns/div" {
			set sampleRate "20MS/s"
			set newBurstCount 11
			set newBurstIncrement 1
			set horizontalBox 50E-9
			set timebaseImage [lindex $timebaseImages 0]
		} "100ns/div" {
			set sampleRate "20MS/s"
			set newBurstCount 21
			set newBurstIncrement 1
			set horizontalBox 100E-9
			set timebaseImage [lindex $timebaseImages 1]
		} "200ns/div" {
			set sampleRate "20MS/s"
			set newBurstCount 41
			set newBurstIncrement 1
			set horizontalBox 200E-9
			set timebaseImage [lindex $timebaseImages 2]
		} "500ns/div" {
			set sampleRate "20MS/s"
			set newBurstCount 101
			set newBurstIncrement 1
			set horizontalBox 500E-9
			set timebaseImage [lindex $timebaseImages 3]
		} "1us/div" {
			set sampleRate "20MS/s"
			#set registerValue [expr {$registerValue + 1}]
			set newBurstCount 201
			set newBurstIncrement 1
			set horizontalBox 1E-6
			set timebaseImage [lindex $timebaseImages 4]
		} "2us/div" {
			set sampleRate "20MS/s"
			#set registerValue [expr {$registerValue + 1}]
			set newBurstCount 401
			set newBurstIncrement 1
			set horizontalBox 2E-6
			set timebaseImage [lindex $timebaseImages 5]
		} "5us/div" {
			set sampleRate "10MS/s"
			#set registerValue [expr {$registerValue + 1}]
			set newBurstCount 501
			set newBurstIncrement 1
			set horizontalBox 5E-6
			set timebaseImage [lindex $timebaseImages 6]
		} "10us/div" {
			set sampleRate "20MS/s"
			#set registerValue [expr {$registerValue +1}]
			set newBurstCount 401
			set newBurstIncrement 5
			set horizontalBox 10E-6
			set timebaseImage [lindex $timebaseImages 7]
		} "20us/div" {
			set sampleRate "10MS/s"
			#set registerValue [expr {$registerValue + 1}]
			set newBurstCount 401
			set newBurstIncrement 5
			set horizontalBox 20E-6
			set timebaseImage [lindex $timebaseImages 8]
		} "50us/div" {
			set sampleRate "20MS/s"
			#set registerValue [expr {$registerValue + 1}]
			set newBurstCount 401
			set newBurstIncrement 25
			set horizontalBox 50E-6
			set timebaseImage [lindex $timebaseImages 9]
		} "100us/div" {
			set sampleRate "10MS/s"
			#set registerValue [expr {$registerValue +2}]
			set newBurstCount 401
			set newBurstIncrement 25
			set horizontalBox 100E-6
			set timebaseImage [lindex $timebaseImages 10]
		} "200us/div" {
			set sampleRate "5MS/s"
			#set registerValue [expr {$registerValue + 2}]
			set newBurstCount 401
			set newBurstIncrement 25
			set horizontalBox 200E-6
			set timebaseImage [lindex $timebaseImages 11]
		} "500us/div" {
			set sampleRate "2.5MS/s"
			#set registerValue [expr {$registerValue + 3}]
			set newBurstCount 501
			set newBurstIncrement 25
			set horizontalBox 500E-6
			set timebaseImage [lindex $timebaseImages 12]
		} "1ms/div" {
			set sampleRate "1.25MS/s"
			#set registerValue [expr {$registerValue + 4}]
			set newBurstCount 501
			set newBurstIncrement 25
			set horizontalBox 1E-3
			set timebaseImage [lindex $timebaseImages 13]
		} "2ms/div" {
			set sampleRate "625kS/s"
			#set registerValue [expr {$registerValue + 5}]
			set newBurstCount 501
			set newBurstIncrement 25
			set horizontalBox 2E-3
			set timebaseImage [lindex $timebaseImages 14]
		} "5ms/div" {
			set sampleRate "312kS/s"
			#set registerValue [expr {$registerValue + 7}]
			set newBurstCount 435
			set newBurstIncrement 36
			set horizontalBox 5E-3
			set timebaseImage [lindex $timebaseImages 15]
		} "10ms/div" {
			set sampleRate "156kS/s"
			#set registerValue [expr {$registerValue + 7}]
			set newBurstCount 435
			set newBurstIncrement 36
			set horizontalBox 10E-3
			set timebaseImage [lindex $timebaseImages 16]
		} "20ms/div" {
			set sampleRate "78kS/s"
			#set registerValue [expr {$registerValue + 8}]
			set newBurstCount 401
			set newBurstIncrement 39
			set horizontalBox 20E-3
			set timebaseImage [lindex $timebaseImages 17]
		} "50ms/div" {
			set sampleRate "19kS/s"
			#set registerValue [expr {$registerValue + 10}]
			set newBurstCount 408
			set newBurstIncrement 24
			set horizontalBox 50E-3
			set timebaseImage [lindex $timebaseImages 18]
		} "100ms/div" {
			set sampleRate "9.7kS/s"
			#set registerValue [expr {$registerValue + 11}]
			set newBurstCount 408
			set newBurstIncrement 24
			set horizontalBox 100E-3
			set timebaseImage [lindex $timebaseImages 19]
		} "200ms/div" {
			set sampleRate "4.9kS/s"
			#set registerValue [expr {$registerValue + 12}]
			set newBurstCount 408
			set newBurstIncrement 24
			set horizontalBox 200E-3
			set timebaseImage [lindex $timebaseImages 20]
		}
	}
	#Add the correct control constant to the register for the sampling rate
	set registerValue [expr $registerValue + [sampleConstant]]
	
	#Update the image on the timebase display label
	.rhs.timebase.mtb configure -image $timebaseImage
	
	#Update the hardware register.  This update is performed
	#now because this register also controls the trigger settings.
	#(A manual trigger requires an immediate change to the int/ext
	#trigger control).
	sendCommand "r$registerValue"
	
	#We can't change the timebase hardware values here in case 
	#the scope is in the middle of a capture.  The values will be
	#sent to the hardware the next time the scope display
	#is updated.  See the  "updateOscilliscope" procedure.
	set updateTimebaseFlag 1
	
	#Update the cursor measurements
	if {$timeCursorsEnable} {
		calculateCursors
	}

}

#Update Oscilloscope
#---------------------
#This is the main procedure which handles all incoming data from
#the hardware and updates the display.  This procedure also updates
#the timebase settings after each capture/screen update cycle.  The
#timebase settings can only be changed after a capture cycle has
#completed.  If the timebase settings are changed mid-capture it
#can cause data loss or misalignment of data between the hardware
#and the GUI.
proc updateOscilloscope { returnData } {
	global autoTriggerHandle	\
		manuallyTriggered	\
		xMid				\
		yPlotEnd			\
		cursorBorder		\
		triggerStateTag		\
		triggerState		\
		enableA			\
		enableB			\
		triggerMode		\
		waveTagA			\
		waveTagB			\
		waveColorA			\
		waveColorB			\
		waveOffsetA		\
		waveOffsetB		\
		dataA			\
		dataB			\
		dataARaw			\
		dataBRaw			\
		updateTimebaseFlag	\
		histoEnabled		\
		xyMode			\
		mathEnabled		\
		xData			\
		exportData			\
		horizontalBox		\
		stepA			\
		stepB				\
		fftEnable			\
		logNow
		
	#Cancel the manual trigger timeout
	after cancel $autoTriggerHandle	
		
	#Determine if this capture cycle was the result of a manual
	#trigger.  If not, make sure the display shows "Triggered".
	if { !$manuallyTriggered} {
		set triggerState "Triggered"
		.display delete $triggerStateTag
		.display create text	\
			[expr {$xMid}] [expr {$yPlotEnd+$cursorBorder/2}]	\
			-fill black	\
			-tag $triggerStateTag	\
			-text $triggerState
	}
	set manuallyTriggered 0
	
	#Process the data received from the scope
	processData $returnData
	
	#Plot the waveforms if the channel is enabled
	if {$enableA} {
		if { $triggerMode != "Hi-Res"} {
			plotWave $dataA $waveColorA $waveOffsetA $waveTagA
		} else {
			plotBrowse $dataARaw $waveColorA browseATag
		}
	}
	if {$enableB} {
		if { $triggerMode != "Hi-Res"} {
			plotWave $dataB $waveColorB $waveOffsetB $waveTagB
		} else {
			plotBrowse $dataBRaw $waveColorB browseBTag
		}
	}
	
	#Save values for export
	set exportData {}
	lappend exportData $xData 
	lappend exportData $dataARaw
	lappend exportData $dataBRaw
	lappend exportData [list $horizontalBox]
	lappend exportData [list $stepA]
	lappend exportData [list $stepB]
	
	if $xyMode {
		plotXY $dataARaw $dataBRaw
	}
	
	#See if any timebase updates are necessary
	if { $updateTimebaseFlag } {
		updateTimebase
		set updateTimebaseFlag 0
	}
	
	if {$histoEnabled} {updateHisto}
	
	if {$mathEnabled && ($triggerMode!="Hi-Res")} {updateMath}
	
	if {$fftEnable} {updateFFT}
	
	if {$logNow} {dataRecord}
	
	autoMeasurements
	
	if {($triggerMode != "Single-Shot") &&  ($triggerMode != "Hi-Res")} {
		acquireWaveform
	}

}

proc updateTimebase {} {
	global burstCount	\
		newBurstCount	\
		burstIncrement	\
		newBurstIncrement	\
		timePos

	#Send the burst count.  First we have to split the
	#burst count into two bytes
	set burstCount $newBurstCount
	set highByte [expr {$burstCount/256}]
	set highByte [expr {floor($highByte)}]
	set highByte [expr {round($highByte)}]
	set lowByte [expr {$burstCount-($highByte*256)}]
	sendCommand "c$lowByte"
	sendCommand "v$highByte"
	
	#Send the burst increment.  Currently, we do not use
	#any burst increment greater than 256.  High byte is
	#included for compatibility.
	set burstIncrement $newBurstIncrement
	#Set burst increment value
	if { $burstIncrement < 256 } {
		sendCommand "l$burstIncrement"
		sendCommand "h0"
	} else {
		error
	}
	
	adjustTimeOffset $timePos
	createXData
}

#Create X Data
#---------------
#When called, this procedure updates a global variable called
#"xData" with a set of X values corresponding to the current
#timebase.  These values are used in conjuction with Y values
#returned from the hardware to plot the waveform.
proc createXData {} {
	global burstCount	\
		xPlotStart		\
		xPlotWidth		\
		xData
	
	set widthTemp [expr {$xPlotWidth*1.0}]
	
	set xData [list]
	for {set i 0} { $i <= [expr {$burstCount-1}]} {incr i} {
		set screenx [expr {($widthTemp/($burstCount-1)) * $i}]
		set screenx [expr {$screenx + $xPlotStart}]
		lappend xData $screenx
	}

}

#Data-to-Screen Coordinates (Y)
#---------------------------------
#This procedure converts 8-bit integers from the hardware
#into the corresponding screen coordinate.
proc datatoscreeny { datay offset dataSource} {
	global yPlotStart		\
		stepA		\
		stepB			\
		verticalBoxB	\
		verticalBoxA	\
		yPlotWidth	
		
	#Figure out which channel we are performing the 
	#conversion for
	if { $dataSource == "a" } {
		set stepSize $stepA
		set verticalBox $verticalBoxA
	} elseif { $dataSource == "b" } {
		set stepSize $stepB
		set verticalBox $verticalBoxB
	}

	#As an intermediate step, we determine the actual
	#real-world voltage of the signal and then map this value
	#on to the screen display
	set actualVoltage [expr {($datay-128)*$stepSize}]
	set numDiv [expr {$actualVoltage/$verticalBox}]
	set screeny [expr {$numDiv*($yPlotWidth/10.0)+($yPlotWidth/2.0)}]
	set screeny [expr {$screeny + $offset}]
	set screeny [expr {int($screeny)}]
	return $screeny
}

#Plot Waveform
#---------------
#This procedure takes data from one of the channels and
#plots it on the screen.  The data must be in the form of a list
#and the values should already have been converted into screen
#coordinates using the "datatoscreeny" procedure.
proc plotWave {yData waveColor waveOffset waveTag} {
	global xData	\
		burstCount	
	
	#Delete the old waveform
	.display delete $waveTag

	#Create a new list for the coordinates of the line and
	#interleave the x and y coordiantes that make up the 
	#waveform.
	set plotData {}
	for {set i 0} {$i <= $burstCount} {incr i} {
		lappend plotData [lindex $xData $i]
		lappend plotData [lindex $yData $i]
	}
	
	#Strip off the empty lists at the end of our plot data
	set plotData [lrange $plotData 0 end-2]

	#Draw the waveform on the display
	.display create line	\
		$plotData		\
		-tag $waveTag	\
		-fill $waveColor	
		

}

#Postscript Export
#------------------
#This procedure exports the canvas to a PostScript file specified
#by the user in a pop-up dialog box.
proc psExport {} {

	set types {
                  {{PostScript}       {.ps}        }
	}
	set filename [tk_getSaveFile \
		-filetypes  $types	\
		-defaultextension ".ps"	\
		-initialfile "scopeCap"	\
		-parent .	\
		-title "Save Waveform..." \
	]

	if {$filename != ""} {
		set psFile [open $filename w]
		.display postscript -channel $psFile
		close $psFile
	}
}

#Toggle Console
#----------------
#This process enables/disables the debug console window.
#Only works with windows.
proc toggleConsole {} {
	global debugConsoleEnable
	
	if {$debugConsoleEnable} {
		console show
	} else {
		console hide
	}
}

#Show the About Box
#---------------------
#Displays a window with version and company info
proc showAbout {} {
	global softwareVersion	\
		companyString	\
		deviceString	\
		firmwareIdent
		
	tk_messageBox \
			-message "$companyString\n$deviceString\nVersion $softwareVersion\n Firmware:\n$firmwareIdent\nhttp://www.syscompdesign.com" \
			-default ok \
			-icon info \
			-title "About"
			
}

#Adjust Timebase
#------------------
#A service routine called when the zoom buttons for the
#timebase are pressed
proc adjustTimebase {direction} {
	global timebaseValues	\
		timebaseIndex	\
		timebaseSetting

	switch $direction {
		"out" {
			incr timebaseIndex
			if {$timebaseIndex > 20} {
				set timebaseIndex 20
			}
		} "in" {
			set timebaseIndex [expr $timebaseIndex -1]
			if {$timebaseIndex <0 } {
				set timebaseIndex 0
			}
		}
	}
	set timebaseSetting [lindex $timebaseValues $timebaseIndex]
	
	adjustSettings
}

#Adjust Vertical Settings
#-------------------------
proc adjustVertical {channel direction} {
	global	verticalIndexA	\
			verticalIndexB	\
			verticalA		\
			verticalB		\
			verticalValues
			
	switch $channel {
		"a" {
			switch $direction {
				"in" {
					set verticalIndexA [expr $verticalIndexA -1]
					if {$verticalIndexA < 0} {
						set verticalIndexA 0
					}
				} "out" {
					incr verticalIndexA
					if {$verticalIndexA > 6} {
						set verticalIndexA 6
					}
				}
			}
		} "b" {
			switch $direction {
				"in" {
					set verticalIndexB [expr $verticalIndexB -1]
					if {$verticalIndexB < 0} {
						set verticalIndexB 0
					}
				} "out" {
					incr verticalIndexB
					if {$verticalIndexB > 6} {
						set verticalIndexB 6
					}
				}
			}
		}
	}
	set verticalA [lindex $verticalValues $verticalIndexA]
	set verticalB [lindex $verticalValues $verticalIndexB]
	setVertical
}

proc selectTriggerMode {} {
	global triggerMode
	
	if {$triggerMode == "Single-Shot" || $triggerMode == "Hi-Res"} {
		.rhs.trigger.singleShot configure -state normal
		if {$triggerMode == "Hi-Res"} {
			selectTimeMode sample
			toggleHRWindow on
		} else {
			selectTimeMode timebase
			toggleHRWindow off
		}
	} else {
		toggleHRWindow off
		.rhs.trigger.singleShot configure -state disabled
		selectTimeMode timebase
		selectChannel
		acquireWaveform
	}
	
}

proc toggleMeasurementsWindow {} {
	global measurementsEnable
	
	if { $measurementsEnable } {
		wm withdraw .measurements
		set measurementsEnable 0
		.rhs.measurementsButton configure -text "Show Measurements"
	} else {
		wm deiconify .measurements
		wm protocol .measurements WM_DELETE_WINDOW {toggleMeasurementsWindow}
		set measurementsEnable 1
		.rhs.measurementsButton configure -text "Hide Measurements"
	}
	
}

proc selectTimeMode {mode} {
	global sampleIntervals	\
		sampleRates	\
		sampleRate		\
		sampleIntervalText	\
		dataCount	\
		unsigned
		
	if {$mode=="sample"} {
		grid remove .rhs.timebase
		grid .rhs.sample -row 1 -column 0 -sticky we
		.display delete timePosCursor
		set sampleIntervalText [lindex $sampleIntervals [lsearch $sampleRates $sampleRate]]
		adjustSampleRate
		#Reset the receive buffers in case there is data left over from
		#another capture (in a different trigger mode)
		set dataCount 0
		set unsigned {}
		.hr.browser delete downloading
		#Initiate a capture
		#acquireWaveform
	} elseif {$mode=="timebase"} {
		grid remove .rhs.sample
		grid .rhs.timebase -row 1 -column 0 -sticky we
		initializeScope
	} else {
		error
	}

}

proc adjustSampleRate {} {
	global intExt		\
		triggerSource	\
		triggerSlope	\
		sampleRate		\
		images		\
		burstCount		\
		burstIncrement	\
		triggerImages
	
	#Initial Value for register (Internal trigger setting)
	if {$intExt} {
		#External Trigger
		set registerValue 0
	} else {
		#Internal Trigger
		set registerValue 16
	}
	
	#Trigger Source
	if { $triggerSource ==  "B" } {
		set registerValue [expr {$registerValue + 64}]
	}
	
	#Trigger Slope
	if { $triggerSlope == "Minus" } {
		set registerValue [expr {$registerValue + 32}]
	}
	
	#Trigger Display
	if {$triggerSource == "A" } {
		if {$triggerSlope == "Plus"} {
			set triggerDisplay [lindex $triggerImages 0]
		} else {
			set triggerDisplay [lindex $triggerImages 1]
		}
	} else {
		if {$triggerSlope == "Plus"} {
			set triggerDisplay [lindex $triggerImages 2]
		} else {
			set triggerDisplay [lindex $triggerImages 3]
		}
	}
	.rhs.trigger.display configure -image $triggerDisplay

	#Set up the sampling rate
	set registerValue [expr $registerValue + [sampleConstant]]
	

	#Update the hardware register.  This update is performed
	#now because this register also controls the trigger settings.
	#(A manual trigger requires an immediate change to the int/ext
	#trigger control).
	sendCommand "r$registerValue"

	#Burst Count
	sendCommand "c0"
	sendCommand "v128"
	set burstCount 32768
	
	#Burst Increment
	sendCommand "l1"
	sendCommand "h0"
	set burstIncrement 1
	
	#Address Offset
	sendCommand "u64"
	sendCommand "d0"

}

proc sampleConstant {} {
	global sampleRate
	
	switch $sampleRate {
		"20MS/s" {
			return 0
			#set registerValue [expr {$registerValue + 0}]
		} "10MS/s" {
			return 1
			#set registerValue [expr {$registerValue + 1}]
		} "5MS/s" {
			return 2
			#set registerValue [expr {$registerValue + 2}]
		} "2.5MS/s" {
			return 3
			#set registerValue [expr {$registerValue + 3}]
		} "1.25MS/s" {
			return 4
			#set registerValue [expr {$registerValue + 4}]
		} "625kS/s" {
			return 5
			#set registerValue [expr {$registerValue + 5}]
		} "312kS/s" {
			return 6
			#set registerValue [expr {$registerValue + 6}]
		} "156kS/s" {
			return 7
			#set registerValue [expr {$registerValue + 7}]
		} "78kS/s" {
			return 8
			#set registerValue [expr {$registerValue + 8}]
		} "39kS/s" {
			return 9
			#set registerValue [expr {$registerValue + 9}]
		} "19kS/s" {
			return 10
			#set registerValue [expr {$registerValue + 10}]
		} "9.7kS/s" {
			return 11
			#set registerValue [expr {$registerValue + 11}]
		} "4.9kS/s" {
			return 12
			#set registerValue [expr {$registerValue + 12}]
		}
	}
	#We should never reach here unless somebody requests an invalid sampling rate
	error "Unknown sampling rate"
	
}

proc togglePlotMode {} {
	global xyMode
	
	if {$xyMode} {
		.display create text	\
			30 423	\
			-fill black	\
			-text "XY Mode"	\
			-tag xyLabel
	} else {
		.display delete xyLabel
		.display delete xyPlotTag
	}

}

proc plotXY { dataA dataB} {
	global yPlotStart		\
		stepA		\
		stepB			\
		verticalBoxB	\
		verticalBoxA	\
		xPlotWidth		\
		yPlotWidth		\
		cursorBorder
	
	.display delete xyPlotTag
	
	set xData {}
	foreach datumA $dataA {
		set actualVoltage [expr {($datumA-128)*$stepA}]
		set numDiv [expr {$actualVoltage/$verticalBoxA}]
		set screenx [expr {$numDiv*($xPlotWidth/10.0)+($xPlotWidth/2.0)}]
		set screenx [expr {$screenx + $cursorBorder}]
		lappend xData $screenx
	}
	
	set yData {}
	foreach datumB $dataB {
		set actualVoltage [expr {($datumB-128)*$stepB}]
		set numDiv [expr {$actualVoltage/$verticalBoxB}]
		set screeny [expr {$numDiv*($yPlotWidth/10.0)+($yPlotWidth/2.0)}]
		set screeny [expr {$screeny + $cursorBorder}]
		lappend yData $screeny
	}
	
	set plotData {}
	foreach xDatum $xData yDatum $yData {
		lappend plotData $xDatum $yDatum
	}
	
	.display create line	\
		$plotData		\
		-tag xyPlotTag
	
}

proc loadSettings {} {
	global verticalA	\
		verticalB	\
		timebaseSetting	\
		triggerSource	\
		enableA		\
		enableB		\
		trigPos		\
		chAGndPos		\
		chBGndPos		\
		yStart		\
		yMid			\
		triggerMode	\
		triggerSlope

	set types {
		{{Config Files}	{.cfg}}
	}

	set settingsFile [tk_getOpenFile -filetypes $types]
	
	if { $settingsFile == ""} {
		return
	} else {
		if {[catch {open $settingsFile r} fileId]} {
			tk_messageBox	\
				-message "Unable to Open Settings File"	\
				-type ok
			loadSettings
			return
		}
	}

	set settings {}
	while { [gets $fileId line] >= 0} {
		lappend settings $line
	}
	
	#Restore the vertical settings
	set verticalA [lindex $settings 0]
	set verticalB [lindex $settings 1]
	setVertical
	set enableA [lindex $settings 2]
	set enableB [lindex $settings 3]
	
	#Restore the timebase settings
	set timebaseSetting [lindex $settings 4]
	set triggerSource [lindex $settings 5]
	set triggerMode [lindex $settings 6]
	set triggerSlope [lindex $settings 7]
	adjustSettings
	
	#Restore the Cursors
	#First we move the trigger cursor in relation to the reference,
	#and then we move the reference
	set trigPos [lindex $settings 8]
	set yStart $yMid
	moveTrigger [expr {$yMid + $trigPos}]
	set chAGndPos [lindex $settings 9]
	set yStart $yMid
	moveChAGnd $chAGndPos
	set chBGndPos [lindex $settings 10]
	set yStart $yMid
	moveChBGnd $chBGndPos

	
	close $fileId
}

proc saveSettings {} {
	global verticalA	\
		verticalB	\
		timebaseSetting	\
		triggerSource	\
		enableA		\
		enableB		\
		triggerLevel		\
		trigPos		\
		chAGndPos		\
		chBGndPos		\
		triggerMode	\
		triggerSlope
	
	set types {
		{{Config Files}	{.cfg}}
	}

	set settingsFile [tk_getSaveFile -filetypes $types]
	
	if {$settingsFile == ""} {return}
	
	if {[catch {open $settingsFile w} fileId]} {
		tk_messageBox	\
			-message "Unable to open file"	\
			-type ok
		saveSettings
		return
	}
	
	#Save the vertical Settings
	puts $fileId $verticalA
	puts $fileId $verticalB
	puts $fileId $enableA
	puts $fileId $enableB
	
	#Save the timebase
	puts $fileId $timebaseSetting
	#Save Trigger Settings
	puts $fileId $triggerSource
	puts $fileId $triggerMode
	puts $fileId $triggerSlope
	
	
	#Save Cursor Settings
	if {$triggerSource == "A"} {
		puts $fileId [expr {$trigPos-$chAGndPos}]
	} else {
		puts $fileId [expr {$trigPos-$chBGndPos}]
	}
	puts $fileId $chAGndPos
	puts $fileId $chBGndPos
	
	close $fileId

}

#======================================
# Creation of Widgets
#======================================

#Main Windows Settings
wm title . "Syscomp USB Digital Storage Oscilloscope DSO-101"
wm iconname . "DSO-101"
wm resizable . 0 0

#Create a frame for the menu bar
frame .menubar -relief raised -borderwidth 1

#Create the Menu Button
menubutton .menubar.file -text "File" \
	-menu .menubar.file.menu
menu .menubar.file.menu -tearoff 0
.menubar.file.menu add command	\
	-label "Save Settings"	\
	-command saveSettings
.menubar.file.menu add command	\
	-label "Load Settings"	\
	-command loadSettings
.menubar.file.menu add separator
.menubar.file.menu add command	\
	-label "Exit"	\
	-command {destroy .}
	
menubutton .menubar.view -text "View"	\
	-menu .menubar.view.menu
menu .menubar.view.menu -tearoff 0
.menubar.view.menu add check \
	-label "Time Cursors" \
	-variable timeCursorsEnable \
	-command toggleTimeCursors
.menubar.view.menu add check \
	-label "Voltage Cursors" \
	-variable voltageCursorsEnable \
	-command toggleVoltageCursors
.menubar.view.menu add check	\
	-label "XY Mode"	\
	-variable xyMode	\
	-command togglePlotMode
.menubar.view.menu add check	\
	-label "Invert Channel A"	\
	-variable invertA
.menubar.view.menu add check	\
	-label "Invert Channel B"	\
	-variable invertB
if {$osType == "windows"} {
	.menubar.view.menu add check \
	-label "Debug Console" \
	-variable debugConsoleEnable \
	-command toggleConsole
}

menubutton .menubar.tools -text "Tools"	\
	-menu .menubar.tools.menu
menu .menubar.tools.menu -tearoff 0
.menubar.tools.menu add command \
	-label "Export to PostScript (PS) File"	\
	-command {psExport}
.menubar.tools.menu add command	\
	-label "Screen Capture (jpg)"	\
	-command {showCapture}
	#-command {windowToFile .}
.menubar.tools.menu add separator
.menubar.tools.menu add command	\
	-label "Calibrate Vertical Sensitivities"	\
	-command {calibration}
.menubar.tools.menu add separator
.menubar.tools.menu add command \
	-label "Math Toolbox"	\
	-command {showMath}
.menubar.tools.menu add separator
	
menubutton .menubar.hardware -text "Hardware"	\
	-menu .menubar.hardware.menu
menu .menubar.hardware.menu -tearoff 0
.menubar.hardware.menu add command \
	-label "Connect"	\
	-command {openSerialPort}
.menubar.hardware.menu add command \
	-label "Port Settings"	\
	-command {showSerialSettings}
.menubar.hardware.menu add separator
.menubar.hardware.menu add checkbutton	\
	-label "Disable Trigger Filtering"	\
	-variable triggerFilter	\
	-command adjustSettings
	
menubutton .menubar.help \
	-text "Help"	\
	-menu .menubar.help.helpmenu
menu .menubar.help.helpmenu -tearoff 0
if { $osType == "windows" } {
	.menubar.help.helpmenu add command	\
		-label "Manual (PDF)"	\
		-command {exec ShelExec.exe manual.pdf &}
} elseif {$osType == "unix" || $osType == "Darwin"} {
   # We look for an application that can open and display a pdf file.
   # We'll take the first available one in the list.
   # If we don't find one, we don't add the menu entry.
   foreach app [list xpdf evince acroread] {
       if { ! [catch {exec which $app}]} {
           .menubar.help.helpmenu add command  \
               -label "Manual (PDF)"   \
               -command [list exec $app manual.pdf &]
           break
       }
   }
}
.menubar.help.helpmenu add command \
	-label "About"	\
	-command showAbout

label .menubar.blanko	\
	-text "           "

label .menubar.serialPortStatus	\
	-textvariable serialStatus	\
	-relief groove	\
	-background red
		
grid .menubar.file -row 0 -column 0
grid .menubar.view -row 0 -column 1
grid .menubar.tools -row 0 -column 2
grid .menubar.hardware -row 0 -column 3
grid .menubar.help -row 0 -column 4
grid .menubar.blanko -row 0 -column 5
grid .menubar.serialPortStatus -row 0 -column 6

#Create a frame for the title and serial port status
frame .titleBar \
	-borderwidth 2
	
label .titleBar.titleLabel \
	-text "Syscomp Oscilloscope"	\
    -text $title   \
    -font {-size 12 -weight bold }
        
#label .titleBar.titleLabel	\
	-image $titleImage
	
grid .titleBar.titleLabel -row 0 -column 0 -sticky w

#Create the frame for the vertical controls
frame .vertical \
    -relief groove  \
    -borderwidth 2

label .vertical.vertLabel1   \
    -text "Vertical Controls" \
    -font {-weight bold -size 10}

#Create a frame for Channel A controls
frame .vertical.cha \
    -relief groove  \
    -borderwidth 3	\
    -padx 3	\
    -pady 3

label .vertical.cha.chaLabel -image $channelAHeader

checkbutton .vertical.cha.enableA   \
    -text "Disable"	\
    -variable enableA	\
    -onvalue 0	\
    -offvalue 1	\
    -command { selectChannel}

button .vertical.cha.zoomOut	\
	-image $zoomOutImage	\
	-command {adjustVertical a out}
	
label .vertical.cha.sensitivity -image $verticalAImage

button .vertical.cha.zoomIn	\
	-image $zoomInImage	\
	-command {adjustVertical a in}


#Place the Channel A Vertical Controls into their frame
grid .vertical.cha.chaLabel -row 0 -column 0 -columnspan 2
grid .vertical.cha.enableA -row 1 -column 0 -columnspan 2
grid .vertical.cha.sensitivity -row 3 -column 0 -columnspan 2
grid .vertical.cha.zoomOut -row 4 -column 0
grid .vertical.cha.zoomIn -row 4 -column 1

#Create a frame for Channel B controls
frame .vertical.chb \
    -relief groove  \
    -borderwidth 3	\
    -padx 3	\
    -pady 3

label .vertical.chb.chbLabel -image $channelBHeader

checkbutton .vertical.chb.enableB \
    -text "Disable"	\
    -variable enableB	\
    -onvalue 0	\
    -offvalue 1	\
    -command { selectChannel}

button .vertical.chb.zoomOut	\
	-image $zoomOutImage	\
	-command {adjustVertical b out}
	
label .vertical.chb.sensitivity -image $verticalBImage

button .vertical.chb.zoomIn	\
	-image $zoomInImage	\
	-command {adjustVertical b in}
    
#Place the Channel B Vertical Controls into their frame
grid .vertical.chb.chbLabel -row 0 -column 0 -columnspan 2
grid .vertical.chb.enableB -row 1 -column 0 -columnspan 2
grid .vertical.chb.sensitivity -row 3 -column 0 -columnspan 2
grid .vertical.chb.zoomOut -row 4 -column 0
grid .vertical.chb.zoomIn -row 4 -column 1

#Place the A & B control frames in the vertical frame
grid .vertical.vertLabel1 -row 0 -column 0
grid .vertical.cha -row 2 -column 0
grid .vertical.chb -row 3 -column 0

#Create a frame for the right-hand side of the screen
#to hold the trigger and timebase controls
frame .rhs

#Create the Frame for the Trigger Controls
frame .rhs.trigger  \
    -relief groove  \
    -borderwidth 2 

label .rhs.trigger.triggerLabel \
    -text "Trigger Controls"    \
    -pady 4 \
    -font {-weight bold -size 10}

label .rhs.trigger.display -image $triggerDisplay

label .rhs.trigger.modeLabel \
	-text "Trigger Mode"	\

#spinbox .rhs.trigger.modes \
	-values $triggerModes	\
	-wrap on	\
	-width 12	\
	-textvariable triggerMode	\
	-command selectTriggerMode

ComboBox .rhs.trigger.modes	\
	-values $triggerModes	\
	-width 12	\
	-textvariable triggerMode	\
	-modifycmd selectTriggerMode	\
	-bwlistbox 1	\
	-hottrack 1		\
	-editable 0
.rhs.trigger.modes setvalue first

button .rhs.trigger.singleShot	\
	-text "Single-Shot Reset"	\
	-command {.display delete $waveTagA; .display delete $waveTagB; acquireWaveform}	\
	-state disabled

label .rhs.trigger.slopeLabel   \
    -text "Trigger Slope"

spinbox .rhs.trigger.slopes \
    -values $triggerSlopes  \
    -wrap on    \
    -width 5		\
    -textvariable triggerSlope	\
    -command { adjustSettings }	\
    -state readonly	\
    -readonlybackground white

label .rhs.trigger.sourceLabel  \
    -text "Trigger Source"

spinbox .rhs.trigger.sources    \
    -values $triggerSources \
    -wrap on    \
    -width 4    \
    -textvariable triggerSource	\
   -command { toggleSource; adjustSettings }	\
   -state readonly	\
   -readonlybackground white
    
button .rhs.trigger.manualTrigger   \
    -text "Manual Trigger"    \
    -command { 
	set intExt 1
	adjustSettings
	sendCommand "m"
	set intExt 0
	adjustSettings
    }

label .rhs.trigger.levelLabel	\
	-text "Trigger Level"
	
label .rhs.trigger.levelValue	\
	-textvariable triggerVoltage	\
	-relief sunken	\
	-width 8

#Place the Trigger Controls
grid .rhs.trigger.triggerLabel -row 0 -column 0
grid .rhs.trigger.display -row 1 -column 0
grid .rhs.trigger.modeLabel -row 2 -column 0
grid .rhs.trigger.modes -row 3 -column 0
grid .rhs.trigger.singleShot -row 4 -column 0
grid .rhs.trigger.slopeLabel -row 5 -column 0
grid .rhs.trigger.slopes -row 6 -column 0
grid .rhs.trigger.sourceLabel -row 7 -column 0
grid .rhs.trigger.sources -row 8 -column 0
grid .rhs.trigger.manualTrigger -row 9 -column 0
grid .rhs.trigger.levelLabel -row 10 -column 0
grid .rhs.trigger.levelValue -row 11 -column 0

#Create the Frame for the Timebase controls
frame .rhs.timebase	\
	-relief groove	\
	-borderwidth 2	
	
label .rhs.timebase.timebaseLabel	\
	-text "Timebase Controls"	\
	-pady 4 \
	-font {-weight bold -size 10}

label .rhs.timebase.horizontalLabel	\
	-text "MTB"

button .rhs.timebase.zoomOut	\
	-image $zoomOutImage	\
	-command {adjustTimebase out}
	
label .rhs.timebase.mtb -image $timebaseImage

button .rhs.timebase.zoomIn	\
	-image $zoomInImage	\
	-command {adjustTimebase in}

grid .rhs.timebase.timebaseLabel -row 0 -column 0 -columnspan 3
grid .rhs.timebase.zoomOut -row 1 -column 0
grid .rhs.timebase.mtb -row 1 -column 1
grid .rhs.timebase.zoomIn -row 1 -column 2

#Create a frame for the Hi-Res Single Shot Sampling Rate Controls
frame .rhs.sample	\
	-relief groove	\
	-borderwidth 2
	
label .rhs.sample.title	\
	-text "Sample Rate Controls"	\
	-pady 4	\
	-font {-weight bold -size 10}
	
label .rhs.sample.rateLabel	\
	-text "Sample Rate:"	\
	-width 15
	
#spinbox .rhs.sample.rate	\
	-values $sampleRates	\
	-wrap on	\
	-width 15	\
	-textvariable sampleRate	\
	-command {adjustSampleRate}

ComboBox .rhs.sample.rate	\
	-values $sampleRates	\
	-width 15	\
	-textvariable sampleRate	\
	-modifycmd adjustSampleRate	\
	-bwlistbox 1	\
	-hottrack 1		\
	-editable 0
.rhs.sample.rate setvalue first

label .rhs.sample.intervalLabel	\
	-text "Sample Interval:"	\
	-width 15
	
label .rhs.sample.interval	\
	-relief sunken	\
	-width 15	\
	-textvariable sampleIntervalText
	
grid .rhs.sample.title -row 0 -column 0 -columnspan 2
grid .rhs.sample.rateLabel -row 1 -column 0
grid .rhs.sample.rate -row 1 -column 1
grid .rhs.sample.intervalLabel -row 2 -column 0
grid .rhs.sample.interval -row 2 -column 1

#Button for hiding/showing the measurements window
button .rhs.measurementsButton	\
	-text "Hide Measurements"	\
	-command toggleMeasurementsWindow

#Place the trigger & timebase controls in the right-hand frame
grid .rhs.trigger -row 0 -column 0
grid .rhs.timebase -row 1 -column 0 -sticky we
grid .rhs.measurementsButton -row 2 -column 0 -sticky we -pady 4

#Create the Screen
canvas .display \
    -width [expr $xPlotWidth + 30]  \
    -height [expr $yPlotWidth + 30] \
    -background white \
    -borderwidth 2

#Create a grid
for {set xpos $xPlotStart} { $xpos < $xPlotEnd } { incr xpos $xDiv10} {
    .display create line $xpos $yPlotStart $xpos $yPlotEnd -fill black -dash . -tag  vertGrid
}
for {set ypos $yPlotStart} { $ypos < $yPlotEnd } {incr ypos $yDiv10} {
    .display create line $xPlotStart $ypos $xPlotEnd $ypos -fill black -dash .
}

#Create the x and y axis
.display create line $xPlotStart $yMid $xPlotEnd $yMid -fill black
.display create line $xMid $yPlotStart $xMid $yPlotEnd -fill black

#Draw horizontal hash marks
for {set xpos $xPlotStart} {$xpos < $xPlotEnd} {incr xpos $xDiv10} {
	.display create line $xpos [expr $yMid-$yHashLength] $xpos [expr $yMid+$yHashLength] -fill black
}
#Draw vertical hash marks
for {set ypos $yPlotStart} {$ypos < $yPlotEnd } {incr ypos $yDiv10} {
	.display create line [expr $xMid-$xHashLength] $ypos [expr $xMid+$xHashLength] $ypos -fill black
}

#Display minor horizontal hash marks
for {set xpos $xPlotStart} {$xpos < $xPlotEnd} {incr xpos [expr ($yDiv10/5)]} {
	.display create line  $xpos [expr $yMid-$yMinorHash] $xpos [expr $yMid+$yMinorHash] -fill black
}
#Display minor vertical hash marks
for {set ypos $yPlotStart} {$ypos <$yPlotEnd} {incr ypos [expr ($yDiv10/5)]} {
	.display create line [expr $xMid-$xMinorHash] $ypos [expr $xMid+$xMinorHash] $ypos -fill black
}

#Place a Frame around the plot area
.display create line $xPlotStart $yPlotStart $xPlotEnd $yPlotStart
.display create line $xPlotStart $yPlotEnd $xPlotEnd $yPlotEnd
.display create line $xPlotStart $yPlotStart $xPlotStart $yPlotEnd
.display create line $xPlotEnd $yPlotStart $xPlotEnd $yPlotEnd

#Frame for cursor and automatic measurements
toplevel .measurements
wm resizable .measurements 0 0
wm title .measurements "Measurements"
wm protocol .measurements WM_DELETE_WINDOW {toggleMeasurementsWindow}

label .measurements.title	\
	-text "Measurements"	\
	-font {-weight bold -size 10}

grid .measurements.title -row 0 -column 0

#Frame for cursor measurements
frame .measurements.cursor \
	-relief groove \
	-borderwidth 2
	
frame .measurements.cursor.freq
frame .measurements.cursor.voltage

label .measurements.cursor.label \
	-text "Cursor Measurements" \
	-font {-weight bold -size 10}

label .measurements.cursor.freq.freqLabel \
	-text "Frequency:" \
	-width 10
	
label .measurements.cursor.freq.freq \
	-textvariable frequency	\
	-relief sunken	\
	-width 12
	
label .measurements.cursor.freq.deltaLabel \
	-text "Delta T:" \
	-width 10
	
label .measurements.cursor.freq.deltaT \
	-textvariable deltaT \
	-width 12 \
	-relief sunken
	
label .measurements.cursor.voltage.vaLabel \
	-text "Delta VA:" \
	-width 10
	
label .measurements.cursor.voltage.deltaVA \
	-textvariable deltaVA \
	-width 12 \
	-relief sunken
	
label .measurements.cursor.voltage.vbLabel \
	-text "Delta VB:" \
	-width 10
	
label .measurements.cursor.voltage.deltaVB \
	-textvariable deltaVB \
	-width 12 \
	-relief sunken

grid .measurements.cursor.label -row 0 -column 0
#grid .cursor.freq -row 1 -column 0
#grid .cursor.voltage -row 2 -column 0

grid .measurements.cursor.freq.freqLabel -row 1 -column 0
grid .measurements.cursor.freq.freq -row 1 -column 1
grid .measurements.cursor.freq.deltaLabel -row 2 -column 0
grid .measurements.cursor.freq.deltaT -row 2 -column 1
grid .measurements.cursor.voltage.vaLabel -row 0 -column 0
grid .measurements.cursor.voltage.deltaVA -row 0 -column 1
grid .measurements.cursor.voltage.vbLabel -row 1 -column 0
grid .measurements.cursor.voltage.deltaVB -row 1 -column 1

#Frame for Automatic Measurements
frame .measurements.auto	\
	-relief groove	\
	-borderwidth 2
	
grid .measurements.auto -row 2 -column 0

label .measurements.auto.autoLabel	\
	-text "Auto Measurements"	\
	-font {-weight bold -size 10}
label .measurements.auto.aLabel	\
	-text "A"	\
	-font {-weight bold -size 10}
label .measurements.auto.bLabel	\
	-text "B"	\
	-font {-weight bold -size 10}
label .measurements.auto.mathLabel	\
	-text "Math"	\
	-font {-weight bold -size 10}

label .measurements.auto.freqLabel	\
	-text "Frequency:"	\
	-width 12
label .measurements.auto.freqA	\
	-relief sunken	\
	-textvariable autoFreqA	\
	-width 12
label .measurements.auto.freqB	\
	-relief sunken	\
	-textvariable autoFreqB	\
	-width 12
label .measurements.auto.freqMath	\
	-relief sunken	\
	-textvariable autoFreqMath	\
	-width 12
label .measurements.auto.freqImage	\
	-image $autoFrequency

label .measurements.auto.periodLabel	\
	-text "Period:"	\
	-width 12
label .measurements.auto.periodA	\
	-relief sunken	\
	-textvariable autoPeriodA	\
	-width 12
label .measurements.auto.periodB	\
	-relief sunken	\
	-textvariable autoPeriodB	\
	-width 12
label .measurements.auto.periodMath	\
	-relief sunken	\
	-textvariable autoPeriodMath	\
	-width 12
	
label .measurements.auto.averageLabel	\
	-text "Average:"	\
	-width 12
label .measurements.auto.averageA	\
	-relief sunken	\
	-textvariable autoAverageA	\
	-width 12
label .measurements.auto.averageB	\
	-relief sunken	\
	-textvariable autoAverageB	\
	-width 12
label .measurements.auto.averageMath	\
	-relief sunken	\
	-textvariable autoAverageMath	\
	-width 12
label .measurements.auto.averageImage	\
	-image $autoAverage
	
label .measurements.auto.maxLabel	\
	-text "Maximum:"	\
	-width 12
label .measurements.auto.maxA	\
	-relief sunken	\
	-textvariable autoMaxA	\
	-width 12
label .measurements.auto.maxB	\
	-relief sunken	\
	-textvariable autoMaxB	\
	-width 12
label .measurements.auto.maxMath	\
	-relief sunken	\
	-textvariable autoMaxMath	\
	-width 12
label .measurements.auto.maxImage	\
	-image $autoMax

label .measurements.auto.minLabel	\
	-text "Minimum:"	\
	-width 12
label .measurements.auto.minA	\
	-relief sunken	\
	-textvariable autoMinA	\
	-width 12
label .measurements.auto.minB	\
	-relief sunken	\
	-textvariable autoMinB	\
	-width 12
label .measurements.auto.minMath	\
	-relief sunken	\
	-textvariable autoMinMath	\
	-width 12
label .measurements.auto.minImage	\
	-image $autoMin
	
label .measurements.auto.pkPkLabel	\
	-text "Peak-Peak:"	\
	-width 12
label .measurements.auto.pkPkA	\
	-relief sunken	\
	-textvariable autoPkPkA	\
	-width 12
label .measurements.auto.pkPkB	\
	-relief sunken	\
	-textvariable autoPkPkB	\
	-width 12
label .measurements.auto.pkPkMath	\
	-relief sunken	\
	-textvariable autoPkPkMath	\
	-width 12
label .measurements.auto.pkPkImage	\
	-image $autoPkPk
	
label .measurements.auto.rmsLabel	\
	-text "RMS:"	\
	-width 12
label .measurements.auto.rmsA	\
	-relief sunken	\
	-textvariable autoRMSA	\
	-width 12
label .measurements.auto.rmsB	\
	-relief sunken	\
	-textvariable autoRMSB	\
	-width 12
label .measurements.auto.rmsMath	\
	-relief sunken	\
	-textvariable autoRMSMath	\
	-width 12
label .measurements.auto.rmsImage	\
	-image $autoRMS

grid .measurements.auto.autoLabel -row 0 -column 0 -columnspan 4
grid .measurements.auto.aLabel -row 1 -column 1
grid .measurements.auto.bLabel -row 1 -column 2

grid .measurements.auto.freqLabel -row 2 -column 0
grid .measurements.auto.freqA -row 2 -column 1
grid .measurements.auto.freqB -row 2 -column 2
grid .measurements.auto.freqImage -row 2 -column 4 -rowspan 2

grid .measurements.auto.periodLabel -row 3 -column 0
grid .measurements.auto.periodA -row 3 -column 1
grid .measurements.auto.periodB -row 3 -column 2

grid .measurements.auto.averageLabel -row 4 -column 0
grid .measurements.auto.averageA -row 4 -column 1
grid .measurements.auto.averageB -row 4 -column 2
grid .measurements.auto.averageImage -row 4 -column 4

grid .measurements.auto.maxLabel -row 5 -column 0
grid .measurements.auto.maxA -row 5 -column 1
grid .measurements.auto.maxB -row 5 -column 2
grid .measurements.auto.maxImage -row 5 -column 4

grid .measurements.auto.minLabel -row 6 -column 0
grid .measurements.auto.minA -row 6 -column 1
grid .measurements.auto.minB -row 6 -column 2
grid .measurements.auto.minImage -row 6 -column 4

grid .measurements.auto.pkPkLabel -row 7 -column 0
grid .measurements.auto.pkPkA -row 7 -column 1
grid .measurements.auto.pkPkB -row 7 -column 2
grid .measurements.auto.pkPkImage -row 7 -column 4

grid .measurements.auto.rmsLabel -row 8 -column 0
grid .measurements.auto.rmsA -row 8 -column 1
grid .measurements.auto.rmsB -row 8 -column 2
grid .measurements.auto.rmsImage -row 8 -column 4

#Place the major frames
grid .menubar -row 0 -column 0 -columnspan 4 -sticky w
grid .titleBar -row 1 -column 0 -columnspan 4 -sticky w
grid .vertical -row 2 -column 0
grid .display -row 2 -column 1
grid .rhs -row 2 -column 2

#Place the ground cursor for Channel A
.display create line \
	$xPlotStart $chAGndPos 	\
	$xPlotEnd $chAGndPos 	\
	-tag chAGnd 	\
	-fill $waveColorA 		\
	-dash .

.display create text 	\
	[expr $xPlotEnd+5] $chAGndPos 	\
	-fill $waveColorA 		\
	-tag chAGnd 	\
	-text "A"

#Place the ground cursor for Channel B
.display create line	\
	$xPlotStart $chBGndPos	\
	$xPlotEnd $chBGndPos	\
	-tag chBGnd	\
	-fill $waveColorB		\
	-dash .	
	
.display create text	\
	[expr $xPlotEnd+10] $chBGndPos	\
	-fill $waveColorB		\
	-tag chBGnd	\
	-text "B"

#Place the trigger cursor
.display create line	\
	$xPlotStart $trigPos	\
	$xPlotEnd $trigPos	\
	-tag trigLevelCursor	\
	-fill green			
.display create text	\
	[expr $xPlotStart - 5] $trigPos	\
	-fill green	\
	-tag trigLevelCursor	\
	-text "T"

#Bind event to cursors
.display bind chAGnd <Button-1> { markYStart chAGnd %y}
.display bind chAGnd <B1-Motion> { moveChAGnd %y }
.display bind chBGnd <Button-1> { markYStart chBGnd %y}
.display bind chBGnd <B1-Motion> { moveChBGnd %y }
.display bind trigLevelCursor <Button-1> { markYStart trigLevelCursor %y }
.display bind trigLevelCursor <B1-Motion> { moveTrigger %y }
.display bind timePosCursor <Button-1> {markXStart timePosCursor %x}
.display bind timePosCursor <B1-Motion> {moveTime %x}

#======================================
# Initialization
#======================================

#Initialize the cursor positions
set yStart $yMid
moveChAGnd $yMid
moveChBGnd $yMid

#Software Defaults
set verticalA "1V/div"
set verticalB "1V/div"
set timebaseSetting "1ms/div"
set horizontalBox 0.001
set enableA 1
set enableB 1

#Create a cache for converting signed integers to
#unsigned ones.  Suggested in comp.lang.tcl in 
#April 11, 2005
set cvt {}
for {set i 0} {$i <256} {incr i} {
	lappend cvt $i
}
set dataCount 0

#Initialize the serial port
if [catch {open scopeport.cfg r+} fileId] {
	if { $osType == "windows" } {
		set serialPort com4
	} elseif {$osType == "unix"} {
		set serialPort "/dev/ttyUSB0"
	}
} else {
	if { [gets $fileId line] >= 0} {
		set serialPort $line
	}
	close $fileId
}

openSerialPort

toggleTimeCursors
toggleVoltageCursors

#======================================
# Add-ons
#======================================

source "$include/histogram.tcl"
source "$include/tooltips.tcl"
source "$include/automeasure.tcl"
source "$include/hi-res.tcl"
source "$include/calibration.tcl"
source "$include/math.tcl"
source "$include/export.tcl"
source "$include/FFT.tcl"
source "$include/dataRecorder.tcl"
source "$include/splash.tcl"