#File: math.tcl
#Syscomp USB Oscilloscope GUI
#Waveform Math Toolbox

#JG
#Copyright 2007 Syscomp Electronic Design
#www.syscompdesign.com

#This program is free software; you can redistribute it and/or
#modify it under the terms of the GNU General Public License as
#published by the Free Software Foundation; either version 2 of
#the License, or (at your option) any later verison.
#
#This program is distributed in the hope that it will be useful, but
#WITHOUT ANY WARRANTY; without even the implied warranty of
#MECHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See
#the GNU General Public License for more details.
#
#You should have received a copy of the GNU General Public License
#along with this program; if not, write to the Free Software
#Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301
#USA

set mathEnabled 0
set mathMode add

set mathHeader [image create photo -file "$images/Math.gif"]
set mathImage [image create photo -file "$images/m1V.gif"]

set verticalBoxM 1.0
set verticalIndexM 7

proc showMath {} {
	global mathEnabled	\
		mathMode		\
		mathHeader	\
		mathImage		\
		zoomOutImage	\
		zoomInImage
	
	if $mathEnabled {wm deiconify .math; raise .math; return}
	
	toplevel .math
	wm title .math "Math Toolbox"
	wm resizable .math 0 0
	bind .math <Destroy> {hideMath}
	
	label .math.title -image $mathHeader
	
	frame .math.mode	\
		-relief groove	\
		-borderwidth 2
	
	radiobutton .math.mode.add		\
		-text "Add: A + B"	\
		-variable mathMode	\
		-value add

	radiobutton .math.mode.subtract		\
		-text "Subtract: A - B"		\
		-variable mathMode	\
		-value subtract
		
	radiobutton .math.mode.multiply	\
		-text "Multiply: A * B"		\
		-variable mathMode		\
		-value multiply

	grid .math.mode.add -row 0 -column 0
	grid .math.mode.subtract -row 1 -column 0
	grid .math.mode.multiply -row 2 -column 0
	
	frame .math.mag	\
		-relief groove	\
		-borderwidth 2
		
	button .math.mag.zoomOut	\
		-image $zoomOutImage	\
		-command {adjustMath out}
		
	button .math.mag.zoomIn	\
		-image $zoomInImage	\
		-command {adjustMath in}
		
	label .math.mag.sensitivity -image $mathImage
	
	grid .math.mag.zoomOut -row 0 -column 0
	grid .math.mag.sensitivity -row 0 -column 1
	grid .math.mag.zoomIn -row 0 -column 2
	
	grid .math.title -row 0 -column 0
	grid .math.mode -row 1 -column 0
	grid .math.mag -row 2 -column 0
	
	grid .measurements.auto.mathLabel -row 1 -column 3
	grid .measurements.auto.freqMath -row 2 -column 3
	grid .measurements.auto.periodMath -row 3 -column 3
	grid .measurements.auto.averageMath -row 4 -column 3
	grid .measurements.auto.maxMath -row 5 -column 3
	grid .measurements.auto.minMath -row 6 -column 3
	grid .measurements.auto.pkPkMath -row 7 -column 3
	grid .measurements.auto.rmsMath -row 8 -column 3
	
	set mathEnabled 1
}

proc hideMath {} {
	global mathEnabled
	
	.display delete math
	
	grid remove .measurements.auto.mathLabel
	grid remove .measurements.auto.freqMath
	grid remove .measurements.auto.periodMath
	grid remove .measurements.auto.averageMath
	grid remove .measurements.auto.maxMath
	grid remove .measurements.auto.minMath
	grid remove .measurements.auto.pkPkMath
	grid remove .measurements.auto.rmsMath
	
	set mathEnabled 0
}

proc updateMath {} {
	global mathMode	\
		yPlotWidth	\
		cursorBorder	\
		verticalBoxM	\
		exportData		\
		mathData

	.display delete math
	
	set xData [lindex $exportData 0]
	set dataA [lindex $exportData 1]
	set dataB [lindex $exportData 2]
	set stepA [lindex $exportData 4]
	set stepB [lindex $exportData 5]
	
	set mathData {}
	
	set voltageA {}
	set voltageB {}
	
	#Compute the real life voltage for the A/D readings
	foreach datumA $dataA datumB $dataB {
		lappend voltageA [expr {(128-$datumA)*$stepA}]
		lappend voltageB [expr {(128-$datumB)*$stepB}]
	}
	
	switch $mathMode {
		"add" {
			foreach datumA $voltageA datumB $voltageB {
				lappend mathData [expr {$datumA + $datumB}]
			}
		} "subtract" {
			foreach datumA $voltageA datumB $voltageB {
				lappend mathData [expr {$datumA - $datumB}]
			}
		} "multiply" {
			foreach datumA $voltageA datumB $voltageB {
				lappend mathData [expr {$datumA * $datumB}]
			}
		}
	}
	
	set plotData {}
	foreach xDatum $xData yDatum $mathData {
		set numDiv [expr {$yDatum/$verticalBoxM}]
		set screeny [expr {($yPlotWidth/2.0)+$cursorBorder-$numDiv*($yPlotWidth/10.0)}]
		lappend plotData $xDatum $screeny
	}
	
	.display create line	\
		$plotData		\
		-tag math		\
		-fill violet
}

proc adjustMath {direction} {
	global verticalIndexM	\
		verticalBoxM	\
		mathImage		\
		mathImages
	
	switch $direction {
		"in" {
			set verticalIndexM [expr {$verticalIndexM - 1}]
		} "out" {
			set verticalIndexM [expr {$verticalIndexM + 1}]
		}
	}
	
	if { $verticalIndexM < 0} { set verticalIndexM 0 }
	if { $verticalIndexM > 12} { set verticalIndexM 12 }
	
	switch $verticalIndexM {
		0 {
			set verticalBoxM 0.005
			set mathImage [lindex $mathImages 0]
		} 1 {
			set verticalBoxM 0.01
			set mathImage [lindex $mathImages 1]
		} 2 {
			set verticalBoxM 0.02
			set mathImage [lindex $mathImages 2]
		} 3 {
			set verticalBoxM 0.05
			set mathImage [lindex $mathImages 3]
		} 4 {
			set verticalBoxM 0.1
			set mathImage [lindex $mathImages 4]
		} 5 {
			set verticalBoxM 0.2
			set mathImage [lindex $mathImages 5]
		} 6 {
			set verticalBoxM 0.5
			set mathImage [lindex $mathImages 6]
		} 7 {
			set verticalBoxM 1.0
			set mathImage [lindex $mathImages 7]
		} 8 {
			set verticalBoxM 2.0
			set mathImage [lindex $mathImages 8]
		} 9 {
			set verticalBoxM 5.0
			set mathImage [lindex $mathImages 9]
		} 10 {
			set verticalBoxM 10.0
			set mathImage [lindex $mathImages 10]
		} 11 {
			set verticalBoxM 20.0
			set mathImage [lindex $mathImages 11]
		} 12 {
			set verticalBoxM 50.0
			set mathImage [lindex $mathImages 12]
		}
	}
	
	.math.mag.sensitivity configure -image $mathImage
	
}

set mathHeader [image create photo -file "$images/Math.gif"]

set mathImages [image create photo -file "$images/m5mV.gif"]
lappend mathImages [image create photo -file "$images/m10mV.gif"]
lappend mathImages [image create photo -file "$images/m20mV.gif"]
lappend mathImages [image create photo -file "$images/m50mV.gif"]
lappend mathImages [image create photo -file "$images/m100mV.gif"]
lappend mathImages [image create photo -file "$images/m200mV.gif"]
lappend mathImages [image create photo -file "$images/m500mV.gif"]
lappend mathImages [image create photo -file "$images/m1V.gif"]
lappend mathImages [image create photo -file "$images/m2V.gif"]
lappend mathImages [image create photo -file "$images/m5V.gif"]
lappend mathImages [image create photo -file "$images/m10V.gif"]
lappend mathImages [image create photo -file "$images/m20V.gif"]
lappend mathImages [image create photo -file "$images/m50V.gif"]
