#File: tooltips.tcl
#Syscomp USB Oscilloscope GUI
#Procedures for creating tooltips associated with widgets

#Reference: Adapted from http://wiki.tcl.tk/534 and
# http://wiki.tcl.tk/1954

#JG
#File Version 1.03 January 2006
#Copyright 2006 Syscomp Electronic Design
#www.syscompdesign.com

#This program is free software; you can redistribute it and/or
#modify it under the terms of the GNU General Public License as
#published by the Free Software Foundation; either version 2 of
#the License, or (at your option) any later verison.
#
#This program is distributed in the hope that it will be useful, but
#WITHOUT ANY WARRANTY; without even the implied warranty of
#MECHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See
#the GNU General Public License for more details.
#
#You should have received a copy of the GNU General Public License
#along with this program; if not, write to the Free Software
#Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301
#USA

#######################################
# Part I - Tool Tip Procedures
# Reference: Adapted from http://wiki.tcl.tk/534 and
# http://wiki.tcl.tk/1954
#######################################

#Assign a New Tooltip
#---------------------
#This procedure binds balloonText to the specified widget.  The
#text will be displayed when the mouse cursor is brought over
#the widget.
proc assignTooltip {widget balloonText} {
	if {$balloonText != ""} {
		bind $widget <Any-Enter> [list after 1000 [list showTooltip %W $balloonText]]
		bind $widget <Any-Leave> {destroy %W.toolTip}
	}
}

proc showTooltip {w arg} {
	if {[eval winfo containing  [winfo pointerxy .]]!=$w} {return}
	set top $w.toolTip
	catch {destroy $top}
	toplevel $top -bd 1 -bg black
	wm overrideredirect $top 1
	if {$::tcl_platform(platform) == "macintosh"} {
		unsupported1 style $top floating sideTitlebar
	}
	pack [message $top.txt -aspect 10000 -bg lightyellow \
		-font fixed -text $arg]
	set wmx [winfo rootx $w]
	set wmy [expr [winfo rooty $w]+[winfo height $w]]
	wm geometry $top \
		[winfo reqwidth $top.txt]x[winfo reqheight $top.txt]+$wmx+$wmy
	raise $top
 }
 
 #######################################
# Part II - Tool Tips
#######################################

#Serial Port Status
assignTooltip .menubar.serialPortStatus "Device connection status"

#Channel A Vertical Controls
assignTooltip .vertical.cha.enableA "Disable channel A on the display"
assignTooltip .vertical.cha.zoomOut "Zoom Out: Decrease vertical sensitivity for channel A"
assignTooltip .vertical.cha.sensitivity "Vertical sensitivity for channel A\n(one vertical division)"
assignTooltip .vertical.cha.zoomIn "Zoom In: Increase vertical sensitivity for channel A"

#Channel B Vertical Controls
assignTooltip .vertical.chb.enableB "Disable channel B on the display"
assignTooltip .vertical.chb.zoomOut "Zoom Out: Decrease vertical sensitivity for channel B"
assignTooltip .vertical.chb.sensitivity "Vertical sensitivity for channel B\n(one vertical division)"
assignTooltip .vertical.chb.zoomIn "Zoom In: Increase vertical sensitivity for channel B"

#Trigger Controls
assignTooltip .rhs.trigger.modes "Trigger Modes:\nAuto: automatically trigger in the absence of a trigger signal\nNormal: only trigger off trigger source\nSingle-Shot: one capture is performed, must be reset manually"
assignTooltip .rhs.trigger.slopes "Plus: trigger off of rising edge\nMinus: trigger off of falling edge"
assignTooltip .rhs.trigger.sources "Scope will trigger off of this signal"
assignTooltip .rhs.trigger.manualTrigger "Trigger the scope now"
assignTooltip .rhs.trigger.levelValue "Trigger voltage level.\nDrag the green T icon on the scope display\nto change the trigger level."
assignTooltip .rhs.trigger.singleShot "Clear display and arm for next trigger"

#Timebase Controls
assignTooltip .rhs.timebase.zoomOut "Zoom out: longer horizontal sweep time"
assignTooltip .rhs.timebase.mtb "Current timebase setting\n(one horizontal division)"
assignTooltip .rhs.timebase.zoomIn "Zoom in: shorter horizontal sweep time"
